@@ -1,13 +1,14 @@
 /* defs.h: Definitions used throughout the program.
  *
- * Copyright (C) 2001-2006 by Brian Raiter, under the GNU General Public
- * License. No warranty. See COPYING for details.
+ * Copyright (C) 2001-2017 by Brian Raiter, Madhav Shanbhag, and Eric Schmidt
+ * under the GNU General Public License. No warranty. See COPYING for details.
  */
 
-#ifndef	_defs_h_
-#define	_defs_h_
+#ifndef	HEADER_defs_h_
+#define	HEADER_defs_h_
 
 #include	<stdio.h>
+#include	<time.h>
 #include	"gen.h"
 
 /*
@@ -20,7 +21,8 @@ enum {
     Ruleset_None = 0,
     Ruleset_Lynx = 1,
     Ruleset_MS = 2,
-    Ruleset_Count
+    Ruleset_Count,
+    Ruleset_First = Ruleset_Lynx
 };
 
 /* File I/O structure.
@@ -43,12 +45,6 @@ typedef	struct prng {
  * Definitions used in game play.
  */
 
-/* Turning macros.
- */
-#define	left(dir)	((((dir) << 1) | ((dir) >> 3)) & 15)
-#define	back(dir)	((((dir) << 2) | ((dir) >> 2)) & 15)
-#define	right(dir)	((((dir) << 3) | ((dir) >> 1)) & 15)
-
 /* A move is specified by its direction and when it takes place.
  */
 typedef	struct action { unsigned int when:23, dir:9; } action;
@@ -61,25 +57,15 @@ typedef struct actlist {
     action	       *list;		/* the array */
 } actlist;
 
-/* A structure holding all the data needed to reconstruct a solution.
- */
-typedef	struct solutioninfo {
-    actlist		moves;		/* the actual moves of the solution */
-    unsigned long	rndseed;	/* the PRNG's initial seed */
-    unsigned long	flags;		/* other flags (currently unused) */
-    unsigned char	rndslidedir;	/* random slide's initial direction */
-    signed char		stepping;	/* the timer offset */
-} solutioninfo;
-
 /* The range of relative mouse moves is a 19x19 square around Chip.
  * (Mouse moves are stored as a relative offset in order to fit all
  * possible moves in nine bits.)
  */
-#define	MOUSERANGEMIN	-9
-#define	MOUSERANGEMAX	+9
+#define	MOUSERANGEMIN	(-9)
+#define	MOUSERANGEMAX	(+9)
 #define	MOUSERANGE	19
 
-/* The complete list of commands that the user can give.
+/* The complete list of commands that the user can given.
  */
 enum {
     CmdNone = NIL,
@@ -110,29 +96,47 @@ enum {
     CmdNext10,
     CmdPauseGame,
     CmdHelp,
+    CmdKeys,
     CmdPlayback,
     CmdCheckSolution,
     CmdReplSolution,
     CmdKillSolution,
     CmdSeeScores,
     CmdSeeSolutionFiles,
+    CmdTimesClipboard,
     CmdVolumeUp,
     CmdVolumeDown,
     CmdStepping,
     CmdSubStepping,
-    CmdRndSlideDir,
+    CmdRandomFF,
+    CmdShowInitState,
+    CmdAdvanceGame,
+    CmdAdvanceMoveGame,
     CmdProceed,
-    CmdPreserve,
-    CmdQuit,
+#ifndef NDEBUG
     CmdDebugCmd1,
     CmdDebugCmd2,
+#endif
+    CmdQuit,
+    CmdPreserve,
+    CmdSeek,
+#ifndef NDEBUG
     CmdCheatNorth,
     CmdCheatWest,
     CmdCheatSouth,
     CmdCheatEast,
     CmdCheatHome,
-    CmdCheatStuff,
-    CmdInvalid
+    CmdCheatKeyRed,
+    CmdCheatKeyBlue,
+    CmdCheatKeyYellow,
+    CmdCheatKeyGreen,
+    CmdCheatBootsIce,
+    CmdCheatBootsSlide,
+    CmdCheatBootsFire,
+    CmdCheatBootsWater,
+    CmdCheatICChip,
+#endif
+    CmdCount
 };
 
 /* True if cmd is a simple directional command, i.e. a single
@@ -193,6 +197,7 @@ typedef	struct gamesetup {
     char const	       *unsolvable;	/* why level is unsolvable, or NULL */
     char		name[256];	/* name of the level */
     char		passwd[256];	/* the level's password */
+    char		author[256];	/* the level's author */
 } gamesetup;
 
 /* Flags associated with a saved game.
@@ -201,13 +206,14 @@ typedef	struct gamesetup {
 #define	SGF_REPLACEABLE		0x0002	/* solution is marked as replaceable */
 #define	SGF_SETNAME		0x0004	/* internal to solution.c */
 
-/* A structure for storing a message text.
+/* The history for the last time a levelset was played.
  */
-typedef struct taggedtext {
-    int		tag;			/* number of the associated level */
-    int		linecount;		/* size of the following array */
-    char      **lines;			/* the text, one paragraph per line */
-} taggedtext;
+typedef	struct history {
+    char		name[256];	/* the set filename minus any path */
+    char		passwd[256];	/* password of the last played level */
+    int	   		levelnumber;	/* number of the last played level */
+    struct tm		dt;		/* date/time set was last played */
+} history;
 
 /* The collection of data maintained for each series.
  */
@@ -222,21 +228,41 @@ typedef	struct gameseries {
     char	       *mapfilename;	/* the name of said file */
     fileinfo		savefile;	/* the file holding the solutions */
     char	       *savefilename;	/* non-default name for said file */
-    taggedtext	       *messages;	/* the set of tagged messages */
-    char	       *msgfilename;	/* the file providing the messages */
-    int			currentlevel;	/* most recently visited level no. */
+    int			solheaderflags;	/* solution flags (none defined yet) */
     int			solheadersize;	/* size of extra solution header */
     char		filebase[256];	/* the level set's filename */
     char		name[256];	/* the filename minus any path */
     unsigned char	solheader[256];	/* extra solution header bytes */
 } gameseries;
 
+/* Just a list of ints */
+typedef struct intlist {
+    int 	*list;		/* intlist */
+    int		 count;		/* size of list */
+} intlist;
+
+/* The possible locations for a levelset.
+ */
+#define LOC_SERIESDIR		0x01
+#define LOC_SERIESDATDIR	0x02
+
+/* Information associated with a levelset. Contains the information about
+ * all gameseries that use the levelset.
+ */
+typedef struct mapfileinfo {
+    char *filename;
+    intlist sfilelst[Ruleset_Count]; /* indices for series files per ruleset */
+    int levelcount;
+    int locdirs;	   /* Flags indicating which dirs the set is in */
+} mapfileinfo;
+
 /* Flags associated with a series.
  */
 #define	GSF_ALLMAPSREAD		0x0001	/* finished reading the data file */
 #define	GSF_NOSAVING		0x0002	/* treat solution file as read-only */
 #define	GSF_NODEFAULTSAVE	0x0004	/* don't use default tws filename */
 #define	GSF_IGNOREPASSWDS	0x0008	/* don't require passwords */
-#define	GSF_LYNXFIXES		0x0010	/* change MS data into Lynx levels */
+#define	GSF_LYNXFIXES		0x0010	/* changes MS data into Lynx levels */
+#define GSF_DATFORDACSERIESDIR	0x0020	/* datfile for .dac is in seriesdir */
 
 #endif
