@@ -1,7 +1,7 @@
 /* fileio.c: Simple file/directory access functions with error-handling.
  *
- * Copyright (C) 2001-2006 by Brian Raiter, under the GNU General Public
- * License. No warranty. See COPYING for details.
+ * Copyright (C) 2001-2017 by Brian Raiter and Eric Schmidt, under the
+ * GNU General Public License. No warranty. See COPYING for details.
  */
 
 #include	<stdio.h>
@@ -9,6 +9,7 @@
 #include	<string.h>
 #include	<errno.h>
 #include	<dirent.h>
+#include	<fcntl.h>
 #include	<sys/types.h>
 #include	<sys/stat.h>
 #include	"err.h"
@@ -46,13 +47,13 @@
 
 /* The function used to display error messages relating to file I/O.
  */
-int _fileerr(char const *cfile, unsigned long lineno,
+int fileerr_(char const *cfile, unsigned long lineno,
 	     fileinfo *file, char const *msg)
 {
     if (msg) {
-	_err_cfile = cfile;
-	_err_lineno = lineno;
-	_errmsg(file->name ? file->name : "file error",
+	err_cfile_ = cfile;
+	err_lineno_ = lineno;
+	errmsg_(file->name ? file->name : "file error",
 		errno ? strerror(errno) : msg);
     }
     return FALSE;
@@ -71,6 +72,23 @@ void clearfileinfo(fileinfo *file)
     file->alloc = FALSE;
 }
 
+/* The 'x' modifier (C11) in fopen() is not widely supported as of 2020.
+ * This hack enables its use regardless of the underlying libc.
+ */
+static FILE *FOPEN(char const *name, char const *mode)
+{
+    FILE * file = NULL;
+    if (!strcmp(mode, "wx")) {
+	int fd = open(name, O_WRONLY | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
+	if (fd != -1)
+	    file = fdopen(fd, "w");
+    }
+    else
+	file = fopen(name, mode);
+
+    return file;
+}
+
 /* Open a file. If the fileinfo structure does not already have a
  * filename assigned to it, use name (after making an independent
  * copy).
@@ -91,7 +109,7 @@ int fileopen(fileinfo *file, char const *name, char const *mode,
 	}
     }
     errno = 0;
-    file->fp = fopen(name, mode);
+    file->fp = FOPEN(name, mode);
     if (file->fp)
 	return TRUE;
     return fileerr(file, msg);
@@ -104,7 +122,7 @@ void fileclose(fileinfo *file, char const *msg)
 {
     errno = 0;
     if (file->fp) {
-	if (!fclose(file->fp))
+	if (fclose(file->fp) == EOF)
 	    fileerr(file, msg);
 	file->fp = NULL;
     }
@@ -115,26 +133,6 @@ void fileclose(fileinfo *file, char const *msg)
     }
 }
 
-/* fgetpos().
- */
-int filegetpos(fileinfo *file, fpos_t *pos, char const *msg)
-{
-    errno = 0;
-    if (!fgetpos(file->fp, pos))
-	return TRUE;
-    return fileerr(file, msg);
-}
-
-/* fsetpos().
- */
-int filesetpos(fileinfo *file, fpos_t *pos, char const *msg)
-{
-    errno = 0;
-    if (!fsetpos(file->fp, pos))
-	return TRUE;
-    return fileerr(file, msg);
-}
-
 /* rewind().
  */
 int filerewind(fileinfo *file, char const *msg)
@@ -187,12 +185,14 @@ void *filereadbuf(fileinfo *file, unsigned long size, char const *msg)
 {
     void       *buf;
 
+    if (size == 0) {
+        return NULL;
+    }
+
     if (!(buf = malloc(size))) {
 	fileerr(file, msg);
 	return NULL;
     }
-    if (!size)
-	return buf;
     errno = 0;
     if (fread(buf, size, 1, file->fp) != 1) {
 	fileerr(file, msg);
@@ -271,9 +271,9 @@ int filereadint16(fileinfo *file, unsigned short *val16, char const *msg)
 
     errno = 0;
     if ((byte = fgetc(file->fp)) != EOF) {
-	*val16 = byte & 0xFFU;
+	*val16 = (unsigned char)byte;
 	if ((byte = fgetc(file->fp)) != EOF) {
-	    *val16 |= (byte & 0xFFU) << 8;
+	    *val16 |= (unsigned char)byte << 8;
 	    return TRUE;
 	}
     }
@@ -299,13 +299,13 @@ int filereadint32(fileinfo *file, unsigned long *val32, char const *msg)
 
     errno = 0;
     if ((byte = fgetc(file->fp)) != EOF) {
-	*val32 = byte & 0xFFUL;
+	*val32 = (unsigned int)byte;
 	if ((byte = fgetc(file->fp)) != EOF) {
-	    *val32 |= (byte & 0xFFUL) << 8;
+	    *val32 |= (unsigned int)byte << 8;
 	    if ((byte = fgetc(file->fp)) != EOF) {
-		*val32 |= (byte & 0xFFUL) << 16;
+		*val32 |= (unsigned int)byte << 16;
 		if ((byte = fgetc(file->fp)) != EOF) {
-		    *val32 |= (byte & 0xFFUL) << 24;
+		    *val32 |= (unsigned int)byte << 24;
 		    return TRUE;
 		}
 	    }
@@ -344,7 +344,7 @@ char *getpathbuffer(void)
 {
     char       *buf;
 
-    if (!(buf = malloc(PATH_MAX + 1)))
+    if (!(buf = calloc(PATH_MAX + 1, 1)))
 	memerrexit();
     return buf;
 }
@@ -473,7 +473,7 @@ int openfileindir(fileinfo *file, char const *dir, char const *filename,
  * contained in it.
  */
 int findfiles(char const *dir, void *data,
-	      int (*filecallback)(char*, void*))
+	      int (*filecallback)(char const*, void*))
 {
     char	       *filename = NULL;
     DIR		       *dp;
@@ -489,7 +489,7 @@ int findfiles(char const *dir, void *data,
     while ((dent = readdir(dp))) {
 	if (dent->d_name[0] == '.')
 	    continue;
-	xalloc(filename, strlen(dent->d_name) + 1);
+	x_alloc(filename, strlen(dent->d_name) + 1);
 	strcpy(filename, dent->d_name);
 	r = (*filecallback)(filename, data);
 	if (r < 0)
