@@ -4,11 +4,15 @@
  * License. No warranty. See COPYING for details.
  */
 
-#ifndef	_fileio_h_
-#define	_fileio_h_
+#ifndef	HEADER_fileio_h_
+#define	HEADER_fileio_h_
 
 #include	"defs.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* Reset a fileinfo structure to indicate no file.
  */
 extern void clearfileinfo(fileinfo *file);
@@ -24,12 +28,12 @@ extern int fileopen(fileinfo *file, char const *name, char const *mode,
 		    char const *msg);
 
 /* The following functions correspond directly to C's standard I/O
- * functions. The extra msg parameter works as described above for
- * fileopen().
+ * functions. If msg is NULL, no error will be displayed if
+ * the operation fails. If msg points to a string, an error will
+ * be displayed. The text of msg will be used only if errno is
+ * zero; otherwise a message appropriate to the error will be used.
  */
 extern int filerewind(fileinfo *file, char const *msg);
-extern int filegetpos(fileinfo *file, fpos_t *pos, char const *msg);
-extern int filesetpos(fileinfo *file, fpos_t *pos, char const *msg);
 extern int fileread(fileinfo *file, void *data, unsigned long size,
 		    char const *msg);
 extern int filewrite(fileinfo *file, void const *data, unsigned long size,
@@ -56,9 +60,9 @@ extern int filewriteint8(fileinfo *file, unsigned char val8,
 extern int filereadint16(fileinfo *file, unsigned short *val16,
 			 char const *msg);
 extern int filewriteint16(fileinfo *file, unsigned short val16,
-			  char const *msg);
-extern int filereadint32(fileinfo *file, unsigned long *val32,
 			 char const *msg);
+extern int filereadint32(fileinfo *file, unsigned long *val32,
+  			 char const *msg);
 extern int filewriteint32(fileinfo *file, unsigned long val32,
 			  char const *msg);
 
@@ -126,15 +130,19 @@ extern int openfileindir(fileinfo *file, char const *dir, char const *filename,
  * examined.
  */
 extern int findfiles(char const *dir, void *data,
-		     int (*filecallback)(char*, void*));
+		     int (*filecallback)(char const*, void*));
 
 /* Display a simple error message prefixed by the name of the given
  * file. If errno is set, a message appropriate to the value is used;
  * otherwise the text pointed to by msg is used. If msg is NULL, the
  * function does nothing. The return value is always FALSE.
  */
-extern int _fileerr(char const *cfile, unsigned long lineno,
+extern int fileerr_(char const *cfile, unsigned long lineno,
 		    fileinfo *file, char const *msg);
-#define	fileerr(file, msg)	(_fileerr(__FILE__, __LINE__, (file), (msg)))
+#define	fileerr(file, msg)	(fileerr_(__FILE__, __LINE__, (file), (msg)))
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif
