@@ -4,8 +4,12 @@
  * License. No warranty. See COPYING for details.
  */
 
-#ifndef	_help_h_
-#define	_help_h_
+#ifndef	HEADER_help_h_
+#define	HEADER_help_h_
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /* The available help topics.
  */
@@ -38,4 +42,8 @@ extern void onlinemainhelp(int topic);
  */
 extern void onlinecontexthelp(int topic);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
