@@ -1,7 +1,7 @@
 /* lxlogic.c: The game logic for the Lynx ruleset.
  *
- * Copyright (C) 2001-2006 by Brian Raiter, under the GNU General Public
- * License. No warranty. See COPYING for details.
+ * Copyright (C) 2001-2015 by Brian Raiter and Eric Schmidt, under the GNU
+ * General Public License. No warranty. See COPYING for details.
  */
 
 #include	<stdlib.h>
@@ -30,14 +30,14 @@
  */
 #define	isdiagonal(dir)	(((dir) & (NORTH | SOUTH)) && ((dir) & (EAST | WEST)))
 
-/* My internal assertion macro.
+/* Internal assertion macro.
  */
 #ifdef NDEBUG
 #define	_assert(test)	((void)0)
 #else
 #define	_assert(test)	((test) || (die("internal error: failed sanity check" \
 				        " (%s)\nPlease report this error to"  \
-				        " breadbox@muppetlabs.com", #test), 0))
+				        " eric41293@comcast.net", #test), 0))
 #endif
 
 /* A list of ways for Chip to lose.
@@ -48,25 +48,10 @@ enum {
     CHIP_NOTOKAY
 };
 
-/* Status information specific to the Lynx game logic.
+/* Pedantic mode flag. (Having this variable defined here is a hack,
+ * but this is the only module that actually uses it.)
  */
-struct lxstate {
-    creature	       *chiptocr;	/* is Chip colliding with a creature */
-    creature	       *crend;		/* near the end of the creature list */
-    short		chiptopos;	/*   just starting to move itself? */
-    unsigned char	prng1;		/* the values used to make the */
-    unsigned char	prng2;		/*   pseudorandom number sequence */
-    signed char		xviewoffset;	/* offset of map view center */
-    signed char		yviewoffset;	/*   position from position of Chip */
-    unsigned char	endgametimer;	/* end-game countdown timer */
-    unsigned char	togglestate;	/* extra state of the toggle walls */
-    unsigned char	completed;	/* level completed successfully */
-    unsigned char	stuck;		/* Chip is stuck on a teleport */
-    unsigned char	pushing;	/* Chip is pushing against something */
-    unsigned char	couldntmove;	/* can't-move sound has been played */
-    unsigned char	mapbreached;	/* border of map has been breached */
-    creature		creaturearray[MAX_CREATURES + 1];  /* all creatures */
-};
+int			pedanticmode = FALSE;
 
 /* Declarations of (indirectly recursive) functions.
  */
@@ -86,6 +71,10 @@ static int		lastrndslidedir = NORTH;
  */
 static int		laststepping = 0;
 
+/* The memory used to hold the list of creatures.
+ */
+static creature	       *creaturearray = NULL;
+
 /* A pointer to the game state, used so that it doesn't have to be
  * passed to every single function.
  */
@@ -121,7 +110,6 @@ static gamestate       *state;
 #define	hidehint()		(state->statusflags &= ~SF_SHOWHINT)
 #define	markinvalid()		(state->statusflags |= SF_INVALID)
 #define	ismarkedinvalid()	(state->statusflags & SF_INVALID)
-#define	ispedanticmode()	(state->statusflags & SF_PEDANTIC)
 
 #define	chipsneeded()		(state->chipsneeded)
 
@@ -130,27 +118,27 @@ static gamestate       *state;
 #define	traplist()		(state->traps)
 #define	traplistsize()		(state->trapcount)
 
-#define	getlxstate()		((struct lxstate*)state->localstateinfo)
+#define	getlxstate()		(state->lxstate)
 
-#define	creaturearray()		(getlxstate()->creaturearray)
-#define	completed()		(getlxstate()->completed)
-#define	togglestate()		(getlxstate()->togglestate)
-#define	couldntmove()		(getlxstate()->couldntmove)
-#define	chippushing()		(getlxstate()->pushing)
-#define	chipstuck()		(getlxstate()->stuck)
-#define	mapbreached()		(getlxstate()->mapbreached)
-#define	chiptopos()		(getlxstate()->chiptopos)
-#define	chiptocr()		(getlxstate()->chiptocr)
-#define	prngvalue1()		(getlxstate()->prng1)
-#define	prngvalue2()		(getlxstate()->prng2)
-#define	xviewoffset()		(getlxstate()->xviewoffset)
-#define	yviewoffset()		(getlxstate()->yviewoffset)
-#define	creaturelistend()	(getlxstate()->crend)
+#define	completed()		(getlxstate().completed)
+#define	togglestate()		(getlxstate().togglestate)
+#define	couldntmove()		(getlxstate().couldntmove)
+#define	chippushing()		(getlxstate().pushing)
+#define	chipstuck()		(getlxstate().stuck)
+#define	mapbreached()		(getlxstate().mapbreached)
+#define	chiptopos()		(getlxstate().chiptopos)
+#define	chiptocr()		(getlxstate().chiptocr)
+#define	putwall()		(getlxstate().putwall)
+#define	prngvalue1()		(getlxstate().prng1)
+#define	prngvalue2()		(getlxstate().prng2)
+#define	xviewoffset()		(getlxstate().xviewoffset)
+#define	yviewoffset()		(getlxstate().yviewoffset)
+#define	creaturelistend()	(getlxstate().crend)
 
-#define	inendgame()		(getlxstate()->endgametimer)
-#define	startendgametimer()	(getlxstate()->endgametimer = 12 + 1)
-#define	decrendgametimer()	(--getlxstate()->endgametimer)
-#define	resetendgametimer()	(getlxstate()->endgametimer = 0)
+#define	inendgame()		(getlxstate().endgametimer)
+#define	startendgametimer()	(getlxstate().endgametimer = 12 + 1)
+#define	decrendgametimer()	(--getlxstate().endgametimer)
+#define	resetendgametimer()	(getlxstate().endgametimer = 0)
 
 #define	addsoundeffect(sfx)	(state->soundeffects |= 1 << (sfx))
 #define	stopsoundeffect(sfx)	(state->soundeffects &= ~(1 << (sfx)))
@@ -214,6 +202,8 @@ static unsigned char lynx_prng(void)
  */
 #define	FS_CLAIMED		0x40	/* spot is claimed by a creature */
 #define	FS_ANIMATED		0x20	/* spot is playing an animation */
+#define FS_BEARTRAP		0x01	/* there is or was a beartrap here */
+#define FS_TELEPORT		0x02	/* there is or was a teleport here */
 
 /* Accessor macros for the floor states.
  */
@@ -223,6 +213,10 @@ static unsigned char lynx_prng(void)
 #define	markanimated(pos)	(state->map[pos].top.state |= FS_ANIMATED)
 #define	clearanimated(pos)	(state->map[pos].top.state &= ~FS_ANIMATED)
 #define	ismarkedanimated(pos)	(state->map[pos].top.state & FS_ANIMATED)
+#define	markbeartrap(pos)	(state->map[pos].top.state |= FS_BEARTRAP)
+#define	ismarkedbeartrap(pos)	(state->map[pos].top.state & FS_BEARTRAP)
+#define	markteleport(pos)	(state->map[pos].top.state |= FS_TELEPORT)
+#define	ismarkedteleport(pos)	(state->map[pos].top.state & FS_TELEPORT)
 
 /* Translate a slide floor into the direction it points in. In the
  * case of a random slide floor, if advance is TRUE a new direction
@@ -277,7 +271,7 @@ static int trapfrombutton(int pos)
     xyconn     *xy;
     int		i;
 
-    if (ispedanticmode()) {
+    if (pedanticmode) {
 	i = pos;
 	for (;;) {
 	    ++i;
@@ -287,6 +281,8 @@ static int trapfrombutton(int pos)
 		break;
 	    if (floorat(i) == Beartrap)
 		return i;
+	    if (ismarkedbeartrap(i))
+	       return -1;
 	}
     } else {
 	for (xy = traplist(), i = traplistsize() ; i ; ++xy, --i)
@@ -303,7 +299,7 @@ static int clonerfrombutton(int pos)
     xyconn     *xy;
     int		i;
 
-    if (ispedanticmode()) {
+    if (pedanticmode) {
 	i = pos;
 	for (;;) {
 	    ++i;
@@ -386,7 +382,7 @@ static creature *newcreature(void)
 	warn("Ran out of room in the creatures array!");
 	return NULL;
     }
-    if (ispedanticmode() && cr - creaturelist() >= PMAX_CREATURES)
+    if (pedanticmode && cr - creaturelist() >= PMAX_CREATURES)
 	return NULL;
 
     cr->hidden = TRUE;
@@ -503,87 +499,175 @@ static void removechip(int reason, creature *also)
  * The laws of movement across the various floors.
  *
  * Chip, blocks, and other creatures all have slightly different rules
- * about what sort of tiles they are permitted to cross. The following
- * lookup table encapsulates these rules. These rules are only a first
- * check; a creature may be generally permitted a particular type of
- * move but still be prevented in a specific situation.
+ * about what sort of tiles they are permitted to move into and out
+ * of. The following lookup table encapsulates these rules. Note that
+ * these rules are only the first check; a creature may be generally
+ * permitted a particular type of move but still prevented in a
+ * specific situation.
  */
 
-#define NWSE	(NORTH | WEST | SOUTH | EAST)
+#define	DIR_IN(dir)	(dir)
+#define	DIR_OUT(dir)	((dir) << 4)
+
+#define	NORTH_IN	DIR_IN(NORTH)
+#define	WEST_IN		DIR_IN(WEST)
+#define	SOUTH_IN	DIR_IN(SOUTH)
+#define	EAST_IN		DIR_IN(EAST)
+#define	NORTH_OUT	DIR_OUT(NORTH)
+#define	WEST_OUT	DIR_OUT(WEST)
+#define	SOUTH_OUT	DIR_OUT(SOUTH)
+#define	EAST_OUT	DIR_OUT(EAST)
+#define	ALL_IN		(NORTH_IN | WEST_IN | SOUTH_IN | EAST_IN)
+#define	ALL_OUT		(NORTH_OUT | WEST_OUT | SOUTH_OUT | EAST_OUT)
+#define	ALL_IN_OUT	(ALL_IN | ALL_OUT)
 
 static struct { unsigned char chip, block, creature; } const movelaws[] = {
-    /* Nothing */		{ 0, 0, 0 },
-    /* Empty */			{ NWSE, NWSE, NWSE },
-    /* Slide_North */		{ NWSE, NWSE, NWSE },
-    /* Slide_West */		{ NWSE, NWSE, NWSE },
-    /* Slide_South */		{ NWSE, NWSE, NWSE },
-    /* Slide_East */		{ NWSE, NWSE, NWSE },
-    /* Slide_Random */		{ NWSE, NWSE, NWSE },
-    /* Ice */			{ NWSE, NWSE, NWSE },
-    /* IceWall_Northwest */	{ SOUTH | EAST, SOUTH | EAST, SOUTH | EAST },
-    /* IceWall_Northeast */	{ SOUTH | WEST, SOUTH | WEST, SOUTH | WEST },
-    /* IceWall_Southwest */	{ NORTH | EAST, NORTH | EAST, NORTH | EAST },
-    /* IceWall_Southeast */	{ NORTH | WEST, NORTH | WEST, NORTH | WEST },
-    /* Gravel */		{ NWSE, NWSE, 0 },
-    /* Dirt */			{ NWSE, 0, 0 },
-    /* Water */			{ NWSE, NWSE, NWSE },
-    /* Fire */			{ NWSE, NWSE, NWSE },
-    /* Bomb */			{ NWSE, NWSE, NWSE },
-    /* Beartrap */		{ NWSE, NWSE, NWSE },
-    /* Burglar */		{ NWSE, 0, 0 },
-    /* HintButton */		{ NWSE, 0, 0 },
-    /* Button_Blue */		{ NWSE, NWSE, NWSE },
-    /* Button_Green */		{ NWSE, NWSE, NWSE },
-    /* Button_Red */		{ NWSE, NWSE, NWSE },
-    /* Button_Brown */		{ NWSE, NWSE, NWSE },
-    /* Teleport */		{ NWSE, NWSE, NWSE },
-    /* Wall */			{ 0, 0, 0 },
-    /* Wall_North */		{ NORTH | WEST | EAST,
-				  NORTH | WEST | EAST,
-				  NORTH | WEST | EAST },
-    /* Wall_West */		{ NORTH | WEST | SOUTH,
-				  NORTH | WEST | SOUTH,
-				  NORTH | WEST | SOUTH },
-    /* Wall_South */		{ WEST | SOUTH | EAST,
-				  WEST | SOUTH | EAST,
-				  WEST | SOUTH | EAST },
-    /* Wall_East */		{ NORTH | SOUTH | EAST,
-				  NORTH | SOUTH | EAST,
-				  NORTH | SOUTH | EAST },
-    /* Wall_Southeast */	{ SOUTH | EAST, SOUTH | EAST, SOUTH | EAST },
-    /* HiddenWall_Perm */	{ 0, 0, 0 },
-    /* HiddenWall_Temp */	{ NWSE, 0, 0 },
-    /* BlueWall_Real */		{ NWSE, 0, 0 },
-    /* BlueWall_Fake */		{ NWSE, 0, 0 },
-    /* SwitchWall_Open */	{ NWSE, NWSE, NWSE },
-    /* SwitchWall_Closed */	{ 0, 0, 0 },
-    /* PopupWall */		{ NWSE, 0, 0 },
-    /* CloneMachine */		{ 0, 0, 0 },
-    /* Door_Red */		{ NWSE, 0, 0 },
-    /* Door_Blue */		{ NWSE, 0, 0 },
-    /* Door_Yellow */		{ NWSE, 0, 0 },
-    /* Door_Green */		{ NWSE, 0, 0 },
-    /* Socket */		{ NWSE, 0, 0 },
-    /* Exit */			{ NWSE, 0, 0 },
-    /* ICChip */		{ NWSE, 0, 0 },
-    /* Key_Red */		{ NWSE, NWSE, NWSE },
-    /* Key_Blue */		{ NWSE, NWSE, NWSE },
-    /* Key_Yellow */		{ NWSE, 0, 0 },
-    /* Key_Green */		{ NWSE, 0, 0 },
-    /* Boots_Slide */		{ NWSE, 0, 0 },
-    /* Boots_Ice */		{ NWSE, 0, 0 },
-    /* Boots_Water */		{ NWSE, 0, 0 },
-    /* Boots_Fire */		{ NWSE, 0, 0 },
-    /* Block_Static */		{ 0, 0, 0 },
-    /* Drowned_Chip */		{ 0, 0, 0 },
-    /* Burned_Chip */		{ 0, 0, 0 },
-    /* Bombed_Chip */		{ 0, 0, 0 },
-    /* Exited_Chip */		{ 0, 0, 0 },
-    /* Exit_Extra_1 */		{ 0, 0, 0 },
-    /* Exit_Extra_2 */		{ 0, 0, 0 },
-    /* Overlay_Buffer */	{ 0, 0, 0 },
-    /* Floor_Reserved2 */	{ 0, 0, 0 },
-    /* Floor_Reserved1 */	{ 0, 0, 0 }
+    /* Nothing */
+    { 0, 0, 0 },
+    /* Empty */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Slide_North */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Slide_West */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Slide_South */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Slide_East */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Slide_Random */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Ice */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* IceWall_Northwest */
+    { NORTH_OUT | WEST_OUT | SOUTH_IN | EAST_IN,
+      NORTH_OUT | WEST_OUT | SOUTH_IN | EAST_IN,
+      NORTH_OUT | WEST_OUT | SOUTH_IN | EAST_IN },
+    /* IceWall_Northeast */
+    { NORTH_OUT | EAST_OUT | SOUTH_IN | WEST_IN,
+      NORTH_OUT | EAST_OUT | SOUTH_IN | WEST_IN,
+      NORTH_OUT | EAST_OUT | SOUTH_IN | WEST_IN },
+    /* IceWall_Southwest */
+    { SOUTH_OUT | WEST_OUT | NORTH_IN | EAST_IN,
+      SOUTH_OUT | WEST_OUT | NORTH_IN | EAST_IN,
+      SOUTH_OUT | WEST_OUT | NORTH_IN | EAST_IN },
+    /* IceWall_Southeast */
+    { SOUTH_OUT | EAST_OUT | NORTH_IN | WEST_IN,
+      SOUTH_OUT | EAST_OUT | NORTH_IN | WEST_IN,
+      SOUTH_OUT | EAST_OUT | NORTH_IN | WEST_IN },
+    /* Gravel */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_OUT },
+    /* Dirt */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Water */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Fire */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Bomb */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Beartrap */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Burglar */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* HintButton */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Button_Blue */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Button_Green */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Button_Red */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Button_Brown */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Teleport */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Wall */
+    { ALL_OUT, ALL_OUT, ALL_OUT },
+    /* Wall_North */
+    { NORTH_IN | WEST_IN | EAST_IN | WEST_OUT | SOUTH_OUT | EAST_OUT,
+      NORTH_IN | WEST_IN | EAST_IN | WEST_OUT | SOUTH_OUT | EAST_OUT,
+      NORTH_IN | WEST_IN | EAST_IN | WEST_OUT | SOUTH_OUT | EAST_OUT },
+    /* Wall_West */
+    { NORTH_IN | WEST_IN | SOUTH_IN | NORTH_OUT | SOUTH_OUT | EAST_OUT,
+      NORTH_IN | WEST_IN | SOUTH_IN | NORTH_OUT | SOUTH_OUT | EAST_OUT,
+      NORTH_IN | WEST_IN | SOUTH_IN | NORTH_OUT | SOUTH_OUT | EAST_OUT },
+    /* Wall_South */
+    { WEST_IN | SOUTH_IN | EAST_IN | NORTH_OUT | WEST_OUT | EAST_OUT,
+      WEST_IN | SOUTH_IN | EAST_IN | NORTH_OUT | WEST_OUT | EAST_OUT,
+      WEST_IN | SOUTH_IN | EAST_IN | NORTH_OUT | WEST_OUT | EAST_OUT },
+    /* Wall_East */
+    { NORTH_IN | SOUTH_IN | EAST_IN | NORTH_OUT | WEST_OUT | SOUTH_OUT,
+      NORTH_IN | SOUTH_IN | EAST_IN | NORTH_OUT | WEST_OUT | SOUTH_OUT,
+      NORTH_IN | SOUTH_IN | EAST_IN | NORTH_OUT | WEST_OUT | SOUTH_OUT },
+    /* Wall_Southeast */
+    { SOUTH_IN | EAST_IN | NORTH_OUT | WEST_OUT,
+      SOUTH_IN | EAST_IN | NORTH_OUT | WEST_OUT,
+      SOUTH_IN | EAST_IN | NORTH_OUT | WEST_OUT },
+    /* HiddenWall_Perm */
+    { ALL_OUT, ALL_OUT, ALL_OUT },
+    /* HiddenWall_Temp */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* BlueWall_Real */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* BlueWall_Fake */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* SwitchWall_Open */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* SwitchWall_Closed */
+    { ALL_OUT, ALL_OUT, ALL_OUT },
+    /* PopupWall */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* CloneMachine */
+    { ALL_OUT, ALL_OUT, ALL_OUT },
+    /* Door_Red */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Door_Blue */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Door_Yellow */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Door_Green */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Socket */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Exit */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* ICChip */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Key_Red */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Key_Blue */
+    { ALL_IN_OUT, ALL_IN_OUT, ALL_IN_OUT },
+    /* Key_Yellow */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Key_Green */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Boots_Slide */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Boots_Ice */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Boots_Water */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Boots_Fire */
+    { ALL_IN_OUT, ALL_OUT, ALL_OUT },
+    /* Block_Static */
+    { 0, 0, 0 },
+    /* Drowned_Chip */
+    { 0, 0, 0 },
+    /* Burned_Chip */
+    { 0, 0, 0 },
+    /* Bombed_Chip */
+    { 0, 0, 0 },
+    /* Exited_Chip */
+    { 0, 0, 0 },
+    /* Exit_Extra_1 */
+    { 0, 0, 0 },
+    /* Exit_Extra_2 */
+    { 0, 0, 0 },
+    /* Overlay_Buffer */
+    { 0, 0, 0 },
+    /* Floor_Reserved2 */
+    { 0, 0, 0 },
+    /* Floor_Reserved1 */
+    { 0, 0, 0 }
 };
 
 /* Including the flag CMM_RELEASING in a call to canmakemove()
@@ -614,8 +698,12 @@ static int canpushblock(creature *block, int dir, int flags)
     _assert(dir != NIL);
 
     if (!canmakemove(block, dir, flags)) {
-	if (!block->moving && (flags & (CMM_PUSHBLOCKS | CMM_PUSHBLOCKSNOW)))
+	if (!block->moving && (flags & (CMM_PUSHBLOCKS | CMM_PUSHBLOCKSNOW))) {
 	    block->dir = dir;
+	    if (pedanticmode) {
+	        block->tdir = dir;
+	    }
+	}
 	return FALSE;
     }
     if (flags & (CMM_PUSHBLOCKS | CMM_PUSHBLOCKSNOW)) {
@@ -672,7 +760,7 @@ static int canmakemove(creature const *cr, int dir, int flags)
     if (x < 0 || x >= CXGRID)
 	return FALSE;
     if (y < 0 || y >= CYGRID) {
-	if (ispedanticmode()) {
+	if (pedanticmode) {
 	    if (flags & CMM_STARTMOVEMENT) {
 		mapbreached() = TRUE;
 		warn("map breach in pedantic mode at (%d %d)", x, y);
@@ -805,8 +893,6 @@ static void choosecreaturemove(creature *cr)
       case Teeth:
 	if ((currenttime() + stepping()) & 4)
 	    return;
-	if (getchip()->hidden)
-	    return;
 	y = chippos() / CXGRID - cr->pos / CXGRID;
 	x = chippos() % CXGRID - cr->pos % CXGRID;
 	n = y < 0 ? NORTH : y > 0 ? SOUTH : NIL;
@@ -858,10 +944,11 @@ static void choosechipmove(creature *cr, int discard)
 
     dir = currentinput();
     currentinput() = NIL;
+
     if (!directionalcmd(dir))
 	dir = NIL;
 
-    if (dir == NIL || discard) {
+    if (dir == NIL || discard || chipstuck()) {
 	cr->tdir = NIL;
 	return;
     }
@@ -915,6 +1002,8 @@ static int getforcedmove(creature *cr)
     if (isice(floor)) {
 	if (cr->id == Chip && possession(Boots_Ice))
 	    return FALSE;
+	if (cr->id == Chip && chipstuck())
+	    return FALSE;
 	if (cr->dir == NIL)
 	    return FALSE;
 	setfdir(cr, cr->dir);
@@ -944,7 +1033,7 @@ static int choosemove(creature *cr)
     } else {
 	if (getforcedmove(cr))
 	    cr->tdir = NIL;
-	else
+	else if (cr->id != Block)
 	    choosecreaturemove(cr);
     }
 
@@ -1004,6 +1093,11 @@ static int teleportcreature(creature *cr)
 		return FALSE;
 	    }
 	}
+	else if (ismarkedteleport(pos)) {
+	    floorat(pos) = Teleport;
+	    if (pos == chippos())
+	        getchip()->hidden = TRUE;
+	}
     }
 
     if (cr->id == Chip)
@@ -1062,7 +1156,7 @@ static void springtrap(int pos)
 	     pos % CXGRID, pos / CXGRID);
 	return;
     }
-    if (floorat(pos) != Beartrap) {
+    if (!ismarkedbeartrap(pos)) {
 	warn("Brown button not connected to a beartrap at (%d %d)",
 	     pos % CXGRID, pos / CXGRID);
 	return;
@@ -1103,7 +1197,6 @@ static int startmovement(creature *cr, int releasing)
     floorfrom = floorat(cr->pos);
 
     if (cr->id == Chip) {
-	_assert(!cr->hidden);
 	if (!possession(Boots_Slide)) {
 	    if (isslide(floorfrom) && cr->tdir == NIL)
 		cr->state |= CS_SLIDETOKEN;
@@ -1129,6 +1222,7 @@ static int startmovement(creature *cr, int releasing)
 	}
 	return 0;
     }
+
     if (mapbreached() && chipisalive()) {
 	removechip(CHIP_COLLIDED, cr);
 	return -1;
@@ -1202,13 +1296,16 @@ static int continuemovement(creature *cr)
 /* Complete the movement of the given creature. Most side effects
  * produced by moving onto a tile occur at this point. FALSE is
  * returned if the creature is removed by the time the function
- * returns.
+ * returns. If stationary is TRUE, we are in pedantic mode and
+ * handling creatures starting on top of something.
  */
-static int endmovement(creature *cr)
+static int endmovement(creature *cr, int stationary)
 {
     int	floor;
     int	survived = TRUE;
 
+    _assert(!stationary || pedanticmode);
+
     if (isanimation(cr->id))
 	return TRUE;
 
@@ -1216,7 +1313,12 @@ static int endmovement(creature *cr)
 
     floor = floorat(cr->pos);
 
-    if (cr->id != Chip || !possession(Boots_Ice))
+    if (cr->id == Chip && putwall() != -1)
+        return TRUE;
+
+    if (cr->id == Chip && !possession(Boots_Ice))
+	applyicewallturn(cr);
+    if (cr->id != Chip && !stationary)
 	applyicewallturn(cr);
 
     if (cr->id == Chip) {
@@ -1228,6 +1330,7 @@ static int endmovement(creature *cr)
 	    }
 	    break;
 	  case Fire:
+	    if (stationary) break;
 	    if (!possession(Boots_Fire)) {
 		removechip(CHIP_BURNED, NULL);
 		survived = FALSE;
@@ -1258,7 +1361,7 @@ static int endmovement(creature *cr)
 	  case Key_Green:
 	    if (possession(floor) == 255)
 		possession(floor) = -1;
-	    /*FALLTHROUGH*/
+	    /* Intentional fall-through */
 	  case Boots_Ice:
 	  case Boots_Slide:
 	  case Boots_Fire:
@@ -1275,13 +1378,14 @@ static int endmovement(creature *cr)
 	    addsoundeffect(SND_BOOTS_STOLEN);
 	    break;
 	  case ICChip:
+	    if (stationary) break;
 	    if (chipsneeded())
 		--chipsneeded();
 	    floorat(cr->pos) = Empty;
 	    addsoundeffect(SND_IC_COLLECTED);
 	    break;
 	  case Socket:
-	    _assert(chipsneeded() == 0);
+	    _assert(stationary || chipsneeded() == 0);
 	    floorat(cr->pos) = Empty;
 	    addsoundeffect(SND_SOCKET_OPENED);
 	    break;
@@ -1323,6 +1427,7 @@ static int endmovement(creature *cr)
 
     switch (floor) {
       case Bomb:
+	if (stationary) break;
 	floorat(cr->pos) = Empty;
 	if (cr->id == Chip) {
 	    removechip(CHIP_BOMBED, NULL);
@@ -1333,23 +1438,35 @@ static int endmovement(creature *cr)
 	survived = FALSE;
 	break;
       case Beartrap:
+      	if (stationary) break;
 	addsoundeffect(SND_TRAP_ENTERED);
 	break;
       case Button_Blue:
+	if (stationary) break;
 	turntanks();
 	addsoundeffect(SND_BUTTON_PUSHED);
 	break;
       case Button_Green:
+	if (stationary) break;
 	togglestate() ^= SwitchWall_Open ^ SwitchWall_Closed;
 	addsoundeffect(SND_BUTTON_PUSHED);
 	break;
       case Button_Red:
+	if (stationary) break;
 	if (activatecloner(clonerfrombutton(cr->pos)))
 	    addsoundeffect(SND_BUTTON_PUSHED);
 	break;
       case Button_Brown:
+	if (stationary) break;
 	addsoundeffect(SND_BUTTON_PUSHED);
 	break;
+      case Socket:
+	_assert(stationary || chipsneeded() == 0);
+	/* Intentional fall-through */
+      case Dirt:
+      case BlueWall_Fake:
+	floorat(cr->pos) = Empty; /* No sound effect */
+	break;
     }
 
     return survived;
@@ -1375,9 +1492,16 @@ static int advancecreature(creature *cr, int releasing)
 	    _assert(cr->dir != NIL);
 	    tdir = cr->tdir;
 	    cr->tdir = cr->dir;
-	} else if (cr->tdir == NIL && getfdir(cr) == NIL)
+	} else if (cr->tdir == NIL && getfdir(cr) == NIL) {
+	    if (pedanticmode && !endmovement(cr, TRUE))
+	        return -1;
 	    return +1;
+	}
 	f = startmovement(cr, releasing);
+	if (f > 0)
+	    cr->hidden = FALSE;
+	if (pedanticmode && f == 0 && !endmovement(cr, TRUE))
+	    return -1;
 	if (f < 0)
 	    return f;
 	if (f == 0) {
@@ -1389,7 +1513,7 @@ static int advancecreature(creature *cr, int releasing)
     }
 
     if (!continuemovement(cr)) {
-	if (!endmovement(cr))
+	if (!endmovement(cr, FALSE))
 	    return -1;
     }
 
@@ -1514,7 +1638,7 @@ static void initialhousekeeping(void)
     }
 
     for (cr = creaturelist() ; cr->id ; ++cr) {
-	if (cr->hidden)
+	if (cr != getchip() && cr->hidden)
 	    continue;
 	if (cr->state & CS_REVERSE) {
 	    cr->state &= ~CS_REVERSE;
@@ -1550,24 +1674,22 @@ static void initialhousekeeping(void)
 	warn("Mark %d (%d).", ++mark, currenttime());
 	currentinput() = NIL;
     }
-    if (currentinput() >= CmdCheatNorth && currentinput() <= CmdCheatStuff) {
+    if (currentinput() >= CmdCheatNorth && currentinput() <= CmdCheatICChip) {
 	switch (currentinput()) {
 	  case CmdCheatNorth:		--yviewoffset();		break;
 	  case CmdCheatWest:		--xviewoffset();		break;
 	  case CmdCheatSouth:		++yviewoffset();		break;
 	  case CmdCheatEast:		++xviewoffset();		break;
 	  case CmdCheatHome:		xviewoffset()=yviewoffset()=0;	break;
-	  case CmdCheatStuff:
-	    possession(Key_Red) = 127;
-	    possession(Key_Blue) = 127;
-	    possession(Key_Yellow) = 127;
-	    possession(Key_Green) = 127;
-	    possession(Boots_Ice) = 127;
-	    possession(Boots_Slide) = 127;
-	    possession(Boots_Fire) = 127;
-	    possession(Boots_Water) = 127;
-	    chipsneeded() = chipsneeded() ? 0 : 1;
-	    break;
+	  case CmdCheatKeyRed:		++possession(Key_Red);		break;
+	  case CmdCheatKeyBlue:		++possession(Key_Blue);		break;
+	  case CmdCheatKeyYellow:	++possession(Key_Yellow);	break;
+	  case CmdCheatKeyGreen:	++possession(Key_Green);	break;
+	  case CmdCheatBootsIce:	++possession(Boots_Ice);	break;
+	  case CmdCheatBootsSlide:	++possession(Boots_Slide);	break;
+	  case CmdCheatBootsFire:	++possession(Boots_Fire);	break;
+	  case CmdCheatBootsWater:	++possession(Boots_Water);	break;
+	  case CmdCheatICChip:	if (chipsneeded()) --chipsneeded();	break;
 	}
 	currentinput() = NIL;
 	setnosaving();
@@ -1615,7 +1737,6 @@ static void preparedisplay(void)
 	    chip->id = Pushing_Chip;
 	if (chip->moving) {
 	    resetfloorsounds(FALSE);
-	    floor = floorat(chip->pos);
 	    if (floor == Fire && possession(Boots_Fire))
 		addsoundeffect(SND_FIREWALKING);
 	    else if (floor == Water && possession(Boots_Water))
@@ -1634,6 +1755,8 @@ static void preparedisplay(void)
 		    addsoundeffect(SND_SLIDING);
 	    }
 	}
+	if (chipstuck() && isice(floor))
+	    addsoundeffect(SND_SKATING_FORWARD);
     }
 }
 
@@ -1656,10 +1779,10 @@ static int initgame(gamelogic *logic)
 
     setstate(logic);
     num = state->game->number;
-    creaturelist() = creaturearray() + 1;
+    creaturelist() = creaturearray + 1;
     cr = creaturelist();
 
-    if (ispedanticmode())
+    if (pedanticmode)
 	if (state->statusflags & SF_BADTILES)
 	    markinvalid();
 
@@ -1671,12 +1794,12 @@ static int initgame(gamelogic *logic)
 	    cell->bot.id = crtile(Block, NORTH);
 	if (ismsspecial(cell->top.id) && cell->top.id != Exited_Chip) {
 	    cell->top.id = Wall;
-	    if (ispedanticmode())
+	    if (pedanticmode)
 		markinvalid();
 	}
 	if (ismsspecial(cell->bot.id) && cell->bot.id != Exited_Chip) {
 	    cell->bot.id = Wall;
-	    if (ispedanticmode())
+	    if (pedanticmode)
 		markinvalid();
 	}
 	if (cell->bot.id != Empty) {
@@ -1691,6 +1814,10 @@ static int initgame(gamelogic *logic)
 	    cr->pos = pos;
 	    cr->id = creatureid(cell->top.id);
 	    cr->dir = creaturedirid(cell->top.id);
+	    if (pedanticmode) {
+	        if (cr->id == Block && isice(cell->bot.id))
+		    cr->dir = NIL;
+	    }
 	    cr->moving = 0;
 	    cr->hidden = FALSE;
 	    if (cr->id == Chip) {
@@ -1712,9 +1839,13 @@ static int initgame(gamelogic *logic)
 	    cell->top.id = cell->bot.id;
 	    cell->bot.id = Empty;
 	}
-	if (ispedanticmode())
+	if (pedanticmode)
 	    if (cell->top.id == Wall_North || cell->top.id == Wall_West)
 		markinvalid();
+	if (cell->top.id == Beartrap)
+	   markbeartrap(pos);
+	if (cell->top.id == Teleport)
+	   markteleport(pos);
     }
 
     if (n < 0) {
@@ -1774,11 +1905,12 @@ static int initgame(gamelogic *logic)
     togglestate() = 0;
     couldntmove() = FALSE;
     chippushing() = FALSE;
-    chipstuck() = FALSE;
+    chipstuck() = (pedanticmode ? isice(floorat(chippos())) : FALSE);
     mapbreached() = FALSE;
     completed() = FALSE;
     chiptopos() = -1;
     chiptocr() = NULL;
+    putwall() = -1;
     prngvalue1() = 0;
     prngvalue2() = 0;
     rndslidedir() = lastrndslidedir;
@@ -1787,6 +1919,7 @@ static int initgame(gamelogic *logic)
     yviewoffset() = 0;
 
     preparedisplay();
+    state->soundeffects = 0;    
     return !ismarkedinvalid();
 }
 
@@ -1801,9 +1934,7 @@ static int advancegame(gamelogic *logic)
     initialhousekeeping();
 
     for (cr = creaturelistend() ; cr >= creaturelist() ; --cr) {
-	setfdir(cr, NIL);
-	cr->tdir = NIL;
-	if (cr->hidden)
+	if (cr != getchip() && cr->hidden)
 	    continue;
 	if (isanimation(cr->id)) {
 	    --cr->frame;
@@ -1811,6 +1942,8 @@ static int advancegame(gamelogic *logic)
 		removeanimation(cr);
 	    continue;
 	}
+	if (cr == getchip() && inendgame())
+	    continue;
 	if (cr->moving <= 0)
 	    choosemove(cr);
     }
@@ -1822,12 +1955,18 @@ static int advancegame(gamelogic *logic)
 	checkmovingto();
 
     for (cr = creaturelistend() ; cr >= creaturelist() ; --cr) {
-	if (cr->hidden)
+    	if (cr == getchip() && completed())
+	    continue;
+	if (cr != getchip() && cr->hidden)
 	    continue;
 	if (advancecreature(cr, FALSE) < 0)
 	    continue;
 	cr->tdir = NIL;
 	setfdir(cr, NIL);
+	if (pedanticmode && floorat(cr->pos) == PopupWall) {
+	    if (cr != getchip())
+	        putwall() = chippos();
+	}
 	if (floorat(cr->pos) == Button_Brown && cr->moving <= 0)
 	    springtrap(trapfrombutton(cr->pos));
     }
@@ -1841,6 +1980,16 @@ static int advancegame(gamelogic *logic)
 	    teleportcreature(cr);
     }
 
+    if (putwall() != -1)
+    {
+	if (!getchip()->hidden) {
+	    if (floorat(chippos()) == Beartrap)
+	        springtrap(chippos());
+	    floorat(putwall()) = Wall;
+	}
+	putwall() = -1;
+    }
+
     finalhousekeeping();
 
     preparedisplay();
@@ -1869,6 +2018,8 @@ static int endgame(gamelogic *logic)
 static void shutdown(gamelogic *logic)
 {
     (void)logic;
+    free(creaturearray);
+    creaturearray = NULL;
 }
 
 /* The exported function: Initialize and return the module's gamelogic
@@ -1878,11 +2029,13 @@ gamelogic *lynxlogicstartup(void)
 {
     static gamelogic	logic;
 
+    creaturearray = calloc(MAX_CREATURES + 1, sizeof *creaturearray);
+    if (!creaturearray)
+	memerrexit();
     lastrndslidedir = NORTH;
     laststepping = 0;
 
     logic.ruleset = Ruleset_Lynx;
-    logic.localstateinfosize = sizeof(struct lxstate);
     logic.initgame = initgame;
     logic.advancegame = advancegame;
     logic.endgame = endgame;
