@@ -1,7 +1,7 @@
 /* sdloshw.c: Top-level SDL management functions.
  *
- * Copyright (C) 2001-2006 by Brian Raiter, under the GNU General Public
- * License. No warranty. See COPYING for details.
+ * Copyright (C) 2001-2010 by Brian Raiter and Madhav Shanbhag,
+ * under the GNU General Public License. No warranty. See COPYING for details.
  */
 
 #include	<stdio.h>
@@ -9,6 +9,7 @@
 #include	<string.h>
 #include	"SDL.h"
 #include	"sdlgen.h"
+#include	"sdlsfx.h"
 #include	"../err.h"
 
 /* Values global to this library.
@@ -24,9 +25,8 @@ oshwglobals	sdlg;
  */
 static void _eventupdate(int wait)
 {
-    static int	mousevisible = TRUE;
+    static int	mouselastx = -1, mouselasty = -1;
     SDL_Event	event;
-    int		x, y;
 
     if (wait)
 	SDL_WaitEvent(NULL);
@@ -34,13 +34,8 @@ static void _eventupdate(int wait)
     while (SDL_PeepEvents(&event, 1, SDL_GETEVENT, SDL_ALLEVENTS)) {
 	switch (event.type) {
 	  case SDL_KEYDOWN:
-	    if (mousevisible) {
-		SDL_GetMouseState(&x, &y);
-		if (windowmappos(x, y) < 0) {
-		    SDL_ShowCursor(SDL_DISABLE);
-		    mousevisible = FALSE;
-		}
-	    }
+	    if (windowmappos(mouselastx, mouselasty) < 0)
+		SDL_ShowCursor(SDL_DISABLE);
 	    keyeventcallback(event.key.keysym.sym, TRUE);
 	    if (event.key.keysym.unicode
 			&& event.key.keysym.unicode != event.key.keysym.sym) {
@@ -49,30 +44,23 @@ static void _eventupdate(int wait)
 	    }
 	    break;
 	  case SDL_KEYUP:
-	    if (mousevisible) {
-		SDL_GetMouseState(&x, &y);
-		if (windowmappos(x, y) < 0) {
-		    SDL_ShowCursor(SDL_DISABLE);
-		    mousevisible = FALSE;
-		}
-	    }
+	    if (windowmappos(mouselastx, mouselasty) < 0)
+		SDL_ShowCursor(SDL_DISABLE);
 	    keyeventcallback(event.key.keysym.sym, FALSE);
 	    break;
 	  case SDL_MOUSEBUTTONDOWN:
 	  case SDL_MOUSEBUTTONUP:
-	    if (!mousevisible) {
-		SDL_ShowCursor(SDL_ENABLE);
-		mousevisible = TRUE;
-	    }
+	    SDL_ShowCursor(SDL_ENABLE);
+	    mouselastx = event.motion.x;
+	    mouselasty = event.motion.y;
 	    mouseeventcallback(event.button.x, event.button.y,
 			       event.button.button,
 			       event.type == SDL_MOUSEBUTTONDOWN);
 	    break;
 	  case SDL_MOUSEMOTION:
-	    if (!mousevisible) {
-		SDL_ShowCursor(SDL_ENABLE);
-		mousevisible = TRUE;
-	    }
+	    SDL_ShowCursor(SDL_ENABLE);
+	    mouselastx = event.motion.x;
+	    mouselasty = event.motion.y;
 	    break;
 	  case SDL_QUIT:
 	    exit(EXIT_SUCCESS);
@@ -80,30 +68,38 @@ static void _eventupdate(int wait)
     }
 }
 
-/* Alter the window decoration, translating the subtitle to UTF-8 as
- * necessary.
+/* Alter the window decoration.
  */
 void setsubtitle(char const *subtitle)
 {
-    char	buf[512] = "Tile World";
-    char       *p;
+    char	buf[270];
 
     if (subtitle && *subtitle) {
-	p = buf + strlen(buf);
-	*p++ = ' ';
-	*p++ = '-';
-	*p++ = ' ';
-	for ( ; *subtitle ; ++subtitle) {
-	    if (*subtitle & 0x80) {
-		*p++ = 0xC0 | ((*subtitle >> 6) & 0x03);
-		*p++ = 0x80 | (*subtitle & 0x3F);
-	    } else {
-		*p++ = *subtitle;
-	    }
-	}
-	*p = '\0';
+	sprintf(buf, "Tile World - %.255s", subtitle);
+	SDL_WM_SetCaption(buf, "Tile World");
+    } else {
+	SDL_WM_SetCaption("Tile World", "Tile World");
     }
-    SDL_WM_SetCaption(buf, "Tile World");
+}
+
+/* Read any additional data for the series.
+ */
+void readextensions(struct gameseries *series)
+{
+    /* Not implemented. */
+}
+
+/* Get number of seconds to skip at start of playback.
+ */
+int getreplaysecondstoskip(void)
+{
+    /* Not implemented. */
+    return -1;
+}
+
+void copytoclipboard(char const *text)
+{
+    /* Not implemented. */
 }
 
 /* Shut down SDL.
@@ -121,7 +117,7 @@ int oshwinitialize(int silence, int soundbufsize,
 {
     SDL_Surface	       *icon;
 
-    sdlg.eventupdatefunc = _eventupdate;
+    geng.eventupdatefunc = _eventupdate;
 
     if (SDL_Init(SDL_INIT_VIDEO) < 0) {
 	errmsg(NULL, "Cannot initialize SDL system: %s\n", SDL_GetError());
@@ -140,9 +136,9 @@ int oshwinitialize(int silence, int soundbufsize,
     } else
 	warn("couldn't create icon surface: %s", SDL_GetError());
 
-    return _sdltimerinitialize(showhistogram)
+    return _generictimerinitialize(showhistogram)
 	&& _sdltextinitialize()
-	&& _sdltileinitialize()
+	&& _generictileinitialize()
 	&& _sdlinputinitialize()
 	&& _sdloutputinitialize(fullscreen)
 	&& _sdlsfxinitialize(silence, soundbufsize);
