@@ -1,15 +1,23 @@
 /* sdlsfx.c: Creating the program's sound effects.
  *
- * Copyright (C) 2001-2006 by Brian Raiter, under the GNU General Public
- * License. No warranty. See COPYING for details.
+ * Copyright (C) 2001-2010 by Brian Raiter and Madhav Shanbhag,
+ * under the GNU General Public License. No warranty. See COPYING for details.
  */
 
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<string.h>
+
+//TWPLUSPLUS really just means TW2
+#ifdef TWPLUSPLUS
+#include <SDL2/SDL.h>
+#else
 #include	"SDL.h"
-#include	"sdlgen.h"
+#endif
+
+#include	"sdlsfx.h"
 #include	"../err.h"
+#include	"../settings.h"
 #include	"../state.h"
 
 /* Some generic default settings for the audio output.
@@ -89,16 +97,11 @@ static void initonomatopoeia(void)
  * Only the first sound is used, since we can't display multiple
  * strings.
  */
-static void displaysoundeffects(unsigned long sfx, int display)
+static void displaysoundeffects(unsigned long sfx)
 {
     unsigned long	flag;
     int			i;
 
-    if (!display) {
-	setdisplaymsg(NULL, 0, 0);
-	return;
-    }
-
     for (flag = 1, i = 0 ; flag ; flag <<= 1, ++i) {
 	if (sfx & flag) {
 	    setdisplaymsg(sounds[i].textsfx, 500, 10);
@@ -266,7 +269,7 @@ void playsoundeffects(unsigned long sfx)
     int			i;
 
     if (!hasaudio || !volume) {
-	displaysoundeffects(sfx, TRUE);
+	displaysoundeffects(sfx);
 	return;
     }
 
@@ -320,13 +323,6 @@ void freesfx(int index)
     }
 }
 
-/* Return the current volume level.
- */
-int getvolume(void)
-{
-    return (10 * volume) / SDL_MIX_MAXVOLUME;
-}
-
 /* Set the current volume level to v. If display is true, the
  * new volume level is displayed to the user.
  */
@@ -340,9 +336,8 @@ int setvolume(int v, int display)
 	v = 0;
     else if (v > 10)
 	v = 10;
-    if (!volume && v)
-	displaysoundeffects(0, FALSE);
     volume = (SDL_MIX_MAXVOLUME * v + 9) / 10;
+    setintsetting("volume", v);
     if (display) {
 	sprintf(buf, "Volume: %d", v);
 	setdisplaymsg(buf, 1000, 1000);
@@ -355,7 +350,7 @@ int setvolume(int v, int display)
  */
 int changevolume(int delta, int display)
 {
-    return setvolume(getvolume() + delta, display);
+    return setvolume(((10 * volume) / SDL_MIX_MAXVOLUME) + delta, display);
 }
 
 /* Shut down the sound system.
@@ -369,16 +364,13 @@ static void shutdown(void)
 }
 
 /* Initialize the module. If silence is TRUE, then the program will
- * leave sound output disabled. Increasing soundbufsize will increase
- * the size of the sound buffer, which decreases noise in the sound
- * effects at the cost of adding latency.
+ * leave sound output disabled.
  */
 int _sdlsfxinitialize(int silence, int _soundbufsize)
 {
     atexit(shutdown);
     enabled = !silence;
-    if (_soundbufsize >= 0 && _soundbufsize < 6)
-	soundbufsize = _soundbufsize;
+    soundbufsize = _soundbufsize;
     initonomatopoeia();
     if (enabled)
 	setaudiosystem(TRUE);
