@@ -1,7 +1,7 @@
 /* sdltext.c: Font-rendering functions for SDL.
  *
- * Copyright (C) 2001-2006 by Brian Raiter, under the GNU General Public
- * License. No warranty. See COPYING for details.
+ * Copyright (C) 2001-2010 by Brian Raiter and Madhav Shanbhag,
+ * under the GNU General Public License. No warranty. See COPYING for details.
  */
 
 #include	<stdio.h>
@@ -23,8 +23,8 @@ static int makefontfromsurface(fontinfo *pf, SDL_Surface *surface)
     unsigned char      *dest;
     Uint8		foregnd, bkgnd;
     int			pitch, wsum;
-    int			x, y, x0, y0, w;
     int			ch;
+    int			x, y, x0, y0, w;
 
     if (surface->format->BytesPerPixel != 1)
 	return FALSE;
@@ -93,8 +93,8 @@ static int makefontfromsurface(fontinfo *pf, SDL_Surface *surface)
     return TRUE;
 }
 
-/* Given a text and a maximum horizontal space to occupy, return the
- * amount of vertical space needed to render the entire text with
+/* Given a text and a maximum horizontal space to occupy, return
+ * the amount of vertial space needed to render the entire text with
  * word-wrapping.
  */
 static int measuremltext(unsigned char const *text, int len, int maxwidth)
@@ -107,11 +107,7 @@ static int measuremltext(unsigned char const *text, int len, int maxwidth)
     brk = 0;
     for (n = 0, w = 0 ; n < len ; ++n) {
 	w += sdlg.font.w[text[n]];
-	if (text[n] == '\n') {
-	    h += sdlg.font.h;
-	    w = 0;
-	    brk = 0;
-	} else if (isspace(text[n])) {
+	if (isspace(text[n])) {
 	    brk = w;
 	} else if (w > maxwidth) {
 	    h += sdlg.font.h;
@@ -258,13 +254,6 @@ static void drawtext(SDL_Rect *rect, unsigned char const *text,
 	rect->w = w;
 	return;
     }
-    if (rect->y <= -sdlg.font.h) {
-	if (flags & PT_UPDATERECT) {
-	    rect->y += sdlg.font.h;
-	    rect->h -= sdlg.font.h;
-	}
-	return;
-    }
     if (w >= rect->w) {
 	w = rect->w;
 	l = r = 0;
@@ -286,9 +275,9 @@ static void drawtext(SDL_Rect *rect, unsigned char const *text,
     else
 	clr = sdlg.textclr.c;
 
-    pitch = sdlg.screen->pitch;
-    bpp = sdlg.screen->format->BytesPerPixel;
-    p = (unsigned char*)sdlg.screen->pixels + rect->y * pitch + rect->x * bpp;
+    pitch = geng.screen->pitch;
+    bpp = geng.screen->format->BytesPerPixel;
+    p = (unsigned char*)geng.screen->pixels + rect->y * pitch + rect->x * bpp;
     for (y = 0 ; y < sdlg.font.h && y < rect->h ; ++y) {
 	switch (bpp) {
 	  case 1:
@@ -328,8 +317,7 @@ static void drawmultilinetext(SDL_Rect *rect, unsigned char const *text,
 			      int len, int flags)
 {
     SDL_Rect	area;
-    int		skip;
-    int		index, newindex, brkw, brkn;
+    int		index, brkw, brkn;
     int		w, n;
 
     if (flags & PT_CALCSIZE) {
@@ -341,36 +329,25 @@ static void drawmultilinetext(SDL_Rect *rect, unsigned char const *text,
 	len = strlen((char const*)text);
 
     area = *rect;
-    skip = flags & PT_SKIPLINES(0xFF);
     brkw = brkn = 0;
-    index = newindex = 0;
+    index = 0;
     for (n = 0, w = 0 ; n < len ; ++n) {
 	w += sdlg.font.w[text[n]];
-	if (text[n] == '\n') {
-	    newindex = n + 1;
-	    brkn = n;
-	    w = 0;
-	} else if (isspace(text[n])) {
+	if (isspace(text[n])) {
 	    brkn = n;
 	    brkw = w;
 	} else if (w > rect->w) {
 	    if (brkw) {
-		newindex = brkn + 1;
+		drawtext(&area, text + index, brkn - index,
+				 flags | PT_UPDATERECT);
+		index = brkn + 1;
 		w -= brkw;
 	    } else {
-		newindex = n;
-		brkn = n;
+		drawtext(&area, text + index, n - index,
+				 flags | PT_UPDATERECT);
+		index = n;
 		w = sdlg.font.w[text[n]];
 	    }
-	}
-	if (newindex) {
-	    if (skip)
-		--skip;
-	    else
-		drawtext(&area, text + index, brkn - index,
-			 flags | PT_UPDATERECT);
-	    index = newindex;
-	    newindex = 0;
 	    brkw = 0;
 	}
     }
@@ -398,16 +375,16 @@ static void _puttext(SDL_Rect *rect, char const *text, int len, int flags)
     if (len < 0)
 	len = text ? strlen(text) : 0;
 
-    if (SDL_MUSTLOCK(sdlg.screen))
-	SDL_LockSurface(sdlg.screen);
+    if (SDL_MUSTLOCK(geng.screen))
+	SDL_LockSurface(geng.screen);
 
     if (flags & PT_MULTILINE)
 	drawmultilinetext(rect, (unsigned char const*)text, len, flags);
     else
 	drawtext(rect, (unsigned char const*)text, len, flags);
 
-    if (SDL_MUSTLOCK(sdlg.screen))
-	SDL_UnlockSurface(sdlg.screen);
+    if (SDL_MUSTLOCK(geng.screen))
+	SDL_UnlockSurface(geng.screen);
 }
 
 /* Lay out the columns of the given table so that the entire table
@@ -456,7 +433,7 @@ static SDL_Rect *_measuretable(SDL_Rect const *area, tablespec const *table)
 	}
     }
 
-    sep = sdlg.font.w['x'] * table->sep;
+    sep = sdlg.font.w[' '] * table->sep;
     w = -sep;
     for (i = 0 ; i < table->cols ; ++i)
 	w += colsizes[i].w + sep;
@@ -464,7 +441,7 @@ static SDL_Rect *_measuretable(SDL_Rect const *area, tablespec const *table)
     if (diff < 0 && table->collapse >= 0) {
 	w = -diff;
 	if (colsizes[table->collapse].w < w)
-	    w = colsizes[table->collapse].w - sdlg.font.w['x'];
+	    w = colsizes[table->collapse].w - sdlg.font.w[' '];
 	colsizes[table->collapse].w -= w;
 	diff += w;
     }
@@ -537,8 +514,8 @@ static int _drawtablerow(tablespec const *table, SDL_Rect *cols,
 	return TRUE;
     }
 
-    if (SDL_MUSTLOCK(sdlg.screen))
-	SDL_LockSurface(sdlg.screen);
+    if (SDL_MUSTLOCK(geng.screen))
+	SDL_LockSurface(geng.screen);
 
     y = cols[0].y;
     n = *row;
@@ -562,8 +539,8 @@ static int _drawtablerow(tablespec const *table, SDL_Rect *cols,
 	    y = rect.y;
     }
 
-    if (SDL_MUSTLOCK(sdlg.screen))
-	SDL_UnlockSurface(sdlg.screen);
+    if (SDL_MUSTLOCK(geng.screen))
+	SDL_UnlockSurface(geng.screen);
 
     *row = n;
     for (i = 0 ; i < table->cols ; ++i) {
