@@ -1,7 +1,7 @@
 /* play.c: Top-level game-playing functions.
  *
- * Copyright (C) 2001-2006 by Brian Raiter, under the GNU General Public
- * License. No warranty. See COPYING for details.
+ * Copyright (C) 2001-2014 by Brian Raiter, Madhav Shanbhag, and Eric Schmidt,
+ * under the GNU General Public License. No warranty. See COPYING for details.
  */
 
 #include	<stdlib.h>
@@ -14,7 +14,9 @@
 #include	"res.h"
 #include	"logic.h"
 #include	"random.h"
+#include	"settings.h"
 #include	"solution.h"
+#include	"unslist.h"
 #include	"play.h"
 
 /* The current state of the current game.
@@ -25,14 +27,19 @@ static gamestate	state;
  */
 static gamelogic       *logic = NULL;
 
-/* TRUE if the user has requested pedantic mode game play.
+/* TRUE if the program is running without a user interface.
  */
-static int		pedanticmode = FALSE;
+int			batchmode = FALSE;
 
 /* How much mud to make the timer suck (i.e., the slowdown factor).
  */
 static int		mudsucking = 1;
 
+/* Whether to show steppping and initial random force floor direction during
+ * solution playback.
+ */
+static int		showinitstate = FALSE;
+
 /* Turn on the pedantry.
  */
 void setpedanticmode(void)
@@ -50,19 +57,23 @@ int setmudsuckingfactor(int mud)
     return TRUE;
 }
 
+void toggleshowinitstate(void)
+{
+    showinitstate = !showinitstate;
+    setintsetting("showinitstate", showinitstate);
+}
+
 /* Configure the game logic, and some of the OS/hardware layer, as
  * required for the given ruleset. Do nothing if the requested ruleset
  * is already the current ruleset.
  */
-static int setrulesetbehavior(int ruleset, int withgui)
+static int setrulesetbehavior(int ruleset)
 {
     if (logic) {
 	if (ruleset == logic->ruleset)
 	    return TRUE;
 	(*logic->shutdown)(logic);
 	logic = NULL;
-	free(state.localstateinfo);
-	state.localstateinfo = NULL;
     }
     if (ruleset == Ruleset_None)
 	return TRUE;
@@ -72,14 +83,16 @@ static int setrulesetbehavior(int ruleset, int withgui)
 	logic = lynxlogicstartup();
 	if (!logic)
 	    return FALSE;
-	setkeyboardarrowsrepeat(TRUE);
+	if (!batchmode)
+	    setkeyboardarrowsrepeat(TRUE);
 	settimersecond(1000 * mudsucking);
 	break;
       case Ruleset_MS:
 	logic = mslogicstartup();
 	if (!logic)
 	    return FALSE;
-	setkeyboardarrowsrepeat(FALSE);
+	if (!batchmode)
+	    setkeyboardarrowsrepeat(FALSE);
 	settimersecond(1100 * mudsucking);
 	break;
       default:
@@ -87,14 +100,13 @@ static int setrulesetbehavior(int ruleset, int withgui)
 	return FALSE;
     }
 
-    if (withgui) {
+    if (!batchmode) {
 	if (!loadgameresources(ruleset) || !creategamedisplay()) {
 	    die("unable to proceed due to previous errors.");
 	    return FALSE;
 	}
     }
 
-    state.localstateinfo = calloc(logic->localstateinfosize, 1);
     logic->state = &state;
     return TRUE;
 }
@@ -102,9 +114,9 @@ static int setrulesetbehavior(int ruleset, int withgui)
 /* Initialize the current state to the starting position of the
  * given level.
  */
-int initgamestate(gamesetup *game, int ruleset, int withgui)
+int initgamestate(gamesetup *game, int ruleset)
 {
-    if (!setrulesetbehavior(ruleset, withgui))
+    if (!setrulesetbehavior(ruleset))
 	die("unable to initialize the system for the requested ruleset");
 
     memset(state.map, 0, sizeof state.map);
@@ -117,11 +129,9 @@ int initgamestate(gamesetup *game, int ruleset, int withgui)
     state.lastmove = NIL;
     state.initrndslidedir = NIL;
     state.stepping = -1;
+    state.statusflags = 0;
     state.soundeffects = 0;
     state.timelimit = game->time * TICKS_PER_SECOND;
-    state.statusflags = 0;
-    if (pedanticmode)
-	state.statusflags |= SF_PEDANTIC;
     initmovelist(&state.moves);
     resetprng(&state.mainprng);
 
@@ -165,57 +175,56 @@ int secondsplayed(void)
 void setgameplaymode(int mode)
 {
     switch (mode) {
-      case BeginInput:
-	setkeyboardinputmode(TRUE);
-	break;
-      case EndInput:
-	setkeyboardinputmode(FALSE);
-	break;
-      case BeginPlay:
-	setkeyboardrepeat(FALSE);
-	settimer(+1);
-	break;
-      case EndPlay:
-	setkeyboardrepeat(TRUE);
-	settimer(-1);
-	break;
-      case BeginVerify:
-	settimer(+1);
-	break;
-      case EndVerify:
-	settimer(-1);
-	break;
-      case SuspendPlayShuttered:
-	if (state.ruleset == Ruleset_MS)
-	    state.statusflags |= SF_SHUTTERED;
-      case SuspendPlay:
-	setkeyboardrepeat(TRUE);
-	settimer(0);
-	setsoundeffects(0);
-	break;
-      case ResumePlay:
+      case NormalPlay:
 	setkeyboardrepeat(FALSE);
 	settimer(+1);
 	setsoundeffects(+1);
 	state.statusflags &= ~SF_SHUTTERED;
 	break;
+      case EndPlay:
+	setkeyboardrepeat(TRUE);
+	settimer(-1);
+	setsoundeffects(+1);
+	break;
+      case NonrenderPlay:
+	settimer(+1);
+	setsoundeffects(0);
+	break;
+      case SuspendPlayShuttered:
+	if (state.ruleset == Ruleset_MS)
+	    state.statusflags |= SF_SHUTTERED;
+	/* fall through */
+      case SuspendPlay:
+	setkeyboardrepeat(TRUE);
+	settimer(0);
+	setsoundeffects(0);
+	break;
     }
 }
 
-/* Alter the stepping. If display is true, the new stepping value is
- * reported to the user.
+/* Write a string representing the stepping to a buffer. Returns
+ * a pointer to the terminating null character.
+ */
+static char *writesteppingstring(char *buf, int stepping)
+{
+    char *p = buf;
+    p += sprintf(p, "%s-step", state.stepping & 4 ? "odd" : "even");
+    if (state.stepping & 3)
+	p += sprintf(p, " +%d", state.stepping & 3);
+    return p;
+}
+
+/* Alter the stepping. If display is true, update the screen to
+ * reflect the change.
  */
 int setstepping(int stepping, int display)
 {
-    char	msg[32], *p;
+    char	msg[32];
 
     state.stepping = stepping;
     if (display) {
-	p = msg;
-	p += sprintf(p, "%s-step", state.stepping & 4 ? "odd" : "even");
-	if (state.stepping & 3)
-	    p += sprintf(p, " +%d", state.stepping & 3);
-	setdisplaymsg(msg, 500, 500);
+	writesteppingstring(msg, stepping);
+	setdisplaymsg(msg, 1000, 1000);
     }
     return TRUE;
 }
@@ -237,31 +246,55 @@ int changestepping(int delta, int display)
     return TRUE;
 }
 
-/* Rotate the initial random slide direction. Note that the stored
- * value for initrndslidedir is actually to the left of the first
- * direction that will actually be used, so the displayed message
- * needs to reflect that.
- */
-int rotaterndslidedir(int display)
-{
-    char	msg[32];
-    char const *dirname;
+/* Append a string literal and update pointer to refer to end. */
+#define APPEND(p, strliteral) do { \
+    strcpy(p, strliteral); \
+    p += strlen(strliteral); \
+    } while (0)
 
-    if (state.ruleset == Ruleset_MS)
-	return FALSE;
-    state.initrndslidedir = right(state.initrndslidedir);
-    if (display) {
-	switch (right(state.initrndslidedir)) {
-	  case NORTH:	dirname = "north";	break;
-	  case WEST:	dirname = "west";	break;
-	  case SOUTH:	dirname = "south";	break;
-	  case EAST:	dirname = "east";	break;
-	  default:	dirname = "(nil)";	break;
-	}
-	sprintf(msg, "random slide: %s", dirname);
-	setdisplaymsg(msg, 500, 500);
+/* Write a string representing the random force floor direction. Returns
+ * a pointer to the terminating null character.
+ */
+static char *writerandomffstring(char *buf, int rndslidedir)
+{
+    char *p = buf;
+    APPEND(p, "random FF ");
+    switch (rndslidedir)
+    {
+	/* The direction will be cycled again before being used */
+	case NORTH: APPEND(p, "east");  break;
+	case EAST:  APPEND(p, "south"); break;
+	case SOUTH: APPEND(p, "west");  break;
+	case WEST:  APPEND(p, "north"); break;
     }
-    return TRUE;
+    return p;
+}
+
+/* Advance initial random force floor direction (in Lynx mode) */
+void advanceinitrandomff(int display)
+{
+    if (state.ruleset == Ruleset_Lynx)
+    {
+        state.initrndslidedir = right(state.initrndslidedir);
+        if (display)
+        {
+            char msg[32];
+	    writerandomffstring(msg, state.initrndslidedir);
+	    setdisplaymsg(msg, 1000, 1000);
+        }       
+    }
+}
+
+char const *getinitstatestring(void)
+{
+    static char buf[64];
+    char *p = buf;
+    p = writesteppingstring(p, state.stepping);
+    if (state.ruleset == Ruleset_Lynx) {
+	*p++ = '\n';
+	p = writerandomffstring(p, state.initrndslidedir);
+    }
+    return buf;
 }
 
 /* Advance the game one tick and update the game state. cmd is the
@@ -310,10 +343,7 @@ int doturn(int cmd)
 	state.lastmove = NIL;
     }
 
-    if (n)
-	return n;
-
-    return 0;
+    return n;
 }
 
 /* Update the display to show the current game state (including sound
@@ -326,27 +356,28 @@ int drawscreen(int showframe)
     int timeleft, besttime;
 
     playsoundeffects(state.soundeffects);
+    state.soundeffects &= ~((1 << SND_ONESHOT_COUNT) - 1);
 
     if (!showframe)
 	return TRUE;
 
     currenttime = state.currenttime + state.timeoffset;
+
+    int const starttime = (state.game->time ? state.game->time : 999);
     if (hassolution(state.game))
-	besttime = (state.game->time ? state.game->time : 999)
-				- state.game->besttime / TICKS_PER_SECOND;
+	besttime = starttime - state.game->besttime / TICKS_PER_SECOND;
     else
 	besttime = TIME_NIL;
 
-    timeleft = TIME_NIL;
-    if (state.game->time) {
-	timeleft = state.game->time - currenttime / TICKS_PER_SECOND;
-	if (timeleft <= 0) {
-	    timeleft = 0;
-	    setdisplaymsg("Out of time", 2, 2);
-	}
+    timeleft = starttime - currenttime / TICKS_PER_SECOND;
+    if (state.game->time && timeleft <= 0) {
+	timeleft = 0;
+#ifndef TWPLUSPLUS
+	setdisplaymsg("Out of time", 2, 2);
+#endif
     }
 
-    return displaygame(&state, timeleft, besttime);
+    return displaygame(&state, timeleft, besttime, showinitstate);
 }
 
 /* Stop game play and clean up.
@@ -370,7 +401,7 @@ int endgamestate(void)
  */
 void shutdowngamestate(void)
 {
-    setrulesetbehavior(Ruleset_None, FALSE);
+    setrulesetbehavior(Ruleset_None);
     destroymovelist(&state.moves);
 }
 
