@@ -1,54 +1,54 @@
 /* play.h: Functions to drive game-play and manage the game state.
  *
- * Copyright (C) 2001-2006 by Brian Raiter, under the GNU General Public
- * License. No warranty. See COPYING for details.
+ * Copyright (C) 2001-2010 by Brian Raiter and Madhav Shanbhag,
+ * under the GNU General Public License. No warranty. See COPYING for details.
  */
 
-#ifndef	_play_h_
-#define	_play_h_
+#ifndef	HEADER_play_h_
+#define	HEADER_play_h_
 
 #include	"defs.h"
 
 /* The different modes of the program with respect to gameplay.
  */
 enum {
-    BeginPlay, EndPlay,
-    SuspendPlay, SuspendPlayShuttered, ResumePlay,
-    BeginInput, EndInput, BeginVerify, EndVerify
+    NormalPlay, EndPlay,
+    SuspendPlay, SuspendPlayShuttered,
+    NonrenderPlay
 };
 
+/* TRUE if the program is running without a user interface.
+ */
+extern int batchmode;
+
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
 /* Change the current gameplay mode. This affects the running of the
  * timer and the handling of the keyboard.
  */
 extern void setgameplaymode(int mode);
 
-/* Initialize the current state to the starting position of the given
- * level. If withgui is FALSE, the user interface is not created.
+/* Initialize the current state to the starting position of the
+ * given level.
  */
-extern int initgamestate(gamesetup *game, int ruleset, int withgui);
+extern int initgamestate(gamesetup *game, int ruleset);
 
 /* Set up the current state to play from its prerecorded solution.
  * FALSE is returned if no solution is available for playback.
  */
 extern int prepareplayback(void);
 
-/* Set the initial stepping value. stepping is a value between 0 and 7
- * inclusive. (Under MS, the stepping can only be 0 or 4.) If display
- * is true, a message is displayed to indicate the change.
- */
 extern int setstepping(int stepping, int display);
-
-/* Change the initial stepping value by adding the given delta. If
- * display is true, the new stepping value is displayed.
- */
 extern int changestepping(int delta, int display);
 
-/* Modify the initial random slide direction by rotating it clockwise
- * If display is true, the new direction is diplayed in a message.
- * FALSE is returned if the current ruleset doesn't use the initial
- * random slide direction.
- */
-extern int rotaterndslidedir(int display);
+extern void advanceinitrandomff(int display);
+
+/* Get a string representing the stepping and (in Lynx mode) initial random
+ * force floor direction. */
+extern char const *getinitstatestring(void);
 
 /* Return the amount of time passed in the current game, in seconds.
  */
@@ -106,9 +106,8 @@ extern int deletesolution(void);
  */
 extern int checksolution(void);
 
-/* Turn pedantic mode on. The ruleset simulation will forgo "standard
- * play" in favor of being as true as possible to the original source
- * material.
+/* Turn pedantic mode on. The ruleset will be slightly changed to be
+ * as faithful as possible to the original source material.
  */
 extern void setpedanticmode(void);
 
@@ -117,4 +116,13 @@ extern void setpedanticmode(void);
  */
 extern int setmudsuckingfactor(int mud);
 
+/* Toggle whether to show stepping/initial random force floor direction
+ * during solution playback.
+ */
+extern void toggleshowinitstate(void);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif
