@@ -1,7 +1,7 @@
 /* res.c: Functions for loading resources from external files.
  *
- * Copyright (C) 2001-2006 by Brian Raiter, under the GNU General Public
- * License. No warranty. See COPYING for details.
+ * Copyright (C) 2001-2014 by Brian Raiter and Eric Schmidt, under the GNU
+ * General Public License. No warranty. See COPYING for details.
  */
 
 #include	<stdlib.h>
@@ -11,6 +11,7 @@
 #include	"fileio.h"
 #include	"err.h"
 #include	"oshw.h"
+#include	"messages.h"
 #include	"unslist.h"
 #include	"res.h"
 
@@ -32,7 +33,8 @@
 
 #define	RES_TXT_BASE		(RES_CLR_LAST + 1)
 #define	RES_TXT_UNSLIST		(RES_TXT_BASE + 0)
-#define	RES_TXT_LAST		RES_TXT_UNSLIST
+#define RES_TXT_MESSAGE		(RES_TXT_BASE + 1)
+#define	RES_TXT_LAST		RES_TXT_MESSAGE
 
 #define	RES_SND_BASE		(RES_TXT_LAST + 1)
 #define	RES_SND_CHIP_LOSES	(RES_SND_BASE + SND_CHIP_LOSES)
@@ -65,46 +67,57 @@
 
 #define	RES_COUNT		(RES_SND_LAST + 1)
 
-/* Place for storing the resource values.
+/* Structure for enumerating the resource names.
  */
-typedef	char	resourceitem[256];
+typedef	struct rcitem {
+    char const *name;
+    int		numeric;
+} rcitem;
+
+/* Union for storing the resource values.
+ */
+typedef union resourceitem {
+    int		num;
+    char	str[256];
+} resourceitem;
 
 /* The complete list of resource names.
  */
-static char const *rclist[] = {
-    "tileimages",
-    "font",
-    "backgroundcolor",
-    "textcolor",
-    "boldtextcolor",
-    "dimtextcolor",
-    "unsolvablelist",
-    "chipdeathsound",
-    "levelcompletesound",
-    "chipdeathbytimesound",
-    "ticksound",
-    "derezzsound",
-    "blockedmovesound",
-    "pickupchipsound",
-    "pickuptoolsound",
-    "thiefsound",
-    "teleportsound",
-    "opendoorsound",
-    "socketsound",
-    "switchsound",
-    "tileemptiedsound",
-    "wallcreatedsound",
-    "trapenteredsound",
-    "bombsound",
-    "splashsound",
-    "blockmovingsound",
-    "skatingforwardsound",
-    "skatingturnsound",
-    "slidingsound",
-    "slidewalkingsound",
-    "icewalkingsound",
-    "waterwalkingsound",
-    "firewalkingsound",
+static rcitem rclist[RES_COUNT] = {
+    { "tileimages",		FALSE },
+    { "font",			FALSE },
+    { "backgroundcolor",	FALSE },
+    { "textcolor",		FALSE },
+    { "boldtextcolor",		FALSE },
+    { "dimtextcolor",		FALSE },
+    { "unsolvablelist",		FALSE },
+    { "endmessages",		FALSE },
+    { "chipdeathsound",		FALSE },
+    { "levelcompletesound",	FALSE },
+    { "chipdeathbytimesound",	FALSE },
+    { "ticksound",		FALSE },
+    { "derezzsound",		FALSE },
+    { "blockedmovesound",	FALSE },
+    { "pickupchipsound",	FALSE },
+    { "pickuptoolsound",	FALSE },
+    { "thiefsound",		FALSE },
+    { "teleportsound",		FALSE },
+    { "opendoorsound",		FALSE },
+    { "socketsound",		FALSE },
+    { "switchsound",		FALSE },
+    { "tileemptiedsound",	FALSE },
+    { "wallcreatedsound",	FALSE },
+    { "trapenteredsound",	FALSE },
+    { "bombsound",		FALSE },
+    { "splashsound",		FALSE },
+    { "blockmovingsound",	FALSE },
+    { "skatingforwardsound",	FALSE },
+    { "skatingturnsound",	FALSE },
+    { "slidingsound",		FALSE },
+    { "slidewalkingsound",	FALSE },
+    { "icewalkingsound",	FALSE },
+    { "waterwalkingsound",	FALSE },
+    { "firewalkingsound",	FALSE }
 };
 
 /* The complete collection of resource values.
@@ -125,23 +138,18 @@ static int		currentruleset = Ruleset_None;
 
 /* The directory containing all the resource files.
  */
-static char const      *resdir = NULL;
-
-/* Getting and setting the resource directory.
- */
-char const *getresdir(void)		{ return resdir; }
-void setresdir(char const *dir)		{ resdir = dir; }
+char		       *resdir = NULL;
 
 /* A few resources have non-empty default values.
  */
 static void initresourcedefaults(void)
 {
-    strcpy(allresources[Ruleset_None][RES_IMG_TILES], "tiles.bmp");
-    strcpy(allresources[Ruleset_None][RES_IMG_FONT], "font.bmp");
-    strcpy(allresources[Ruleset_None][RES_CLR_BKGND], "000000");
-    strcpy(allresources[Ruleset_None][RES_CLR_TEXT], "FFFFFF");
-    strcpy(allresources[Ruleset_None][RES_CLR_BOLD], "FFFF00");
-    strcpy(allresources[Ruleset_None][RES_CLR_DIM], "C0C0C0");
+    strcpy(allresources[Ruleset_None][RES_IMG_TILES].str, "tiles.bmp");
+    strcpy(allresources[Ruleset_None][RES_IMG_FONT].str, "font.bmp");
+    strcpy(allresources[Ruleset_None][RES_CLR_BKGND].str, "000000");
+    strcpy(allresources[Ruleset_None][RES_CLR_TEXT].str, "FFFFFF");
+    strcpy(allresources[Ruleset_None][RES_CLR_BOLD].str, "FFFF00");
+    strcpy(allresources[Ruleset_None][RES_CLR_DIM].str, "C0C0C0");
     memcpy(&allresources[Ruleset_MS], globalresources,
 				sizeof allresources[Ruleset_MS]);
     memcpy(&allresources[Ruleset_Lynx], globalresources,
@@ -159,14 +167,13 @@ static void initresourcedefaults(void)
 static int readrcfile(void)
 {
     resourceitem	item;
-    fileinfo		file;
+    fileinfo		file = {0};
     char		buf[256];
     char		name[256];
     char	       *p;
     int			ruleset;
     int			lineno, i, j;
 
-    memset(&file, 0, sizeof file);
     if (!openfileindir(&file, resdir, "rc", "r", "can't open"))
 	return FALSE;
 
@@ -190,22 +197,26 @@ static int readrcfile(void)
 		warn("rc:%d: syntax error", lineno);
 	    continue;
 	}
-	if (sscanf(buf, "%[^=]=%s", name, item) != 2) {
+	if (sscanf(buf, "%[^=]=%s", name, item.str) != 2) {
 	    warn("rc:%d: syntax error", lineno);
 	    continue;
 	}
 	for (p = name ; (*p = tolower(*p)) != '\0' ; ++p) ;
 	for (i = sizeof rclist / sizeof *rclist - 1 ; i >= 0 ; --i)
-	    if (!strcmp(name, rclist[i]))
+	    if (!strcmp(name, rclist[i].name))
 		break;
 	if (i < 0) {
 	    warn("rc:%d: illegal resource name \"%s\"", lineno, name);
 	    continue;
 	}
-	strcpy(allresources[ruleset][i], item);
+	if (rclist[i].numeric) {
+	    i = atoi(item.str);
+	    item.num = i;
+	}
+	allresources[ruleset][i] = item;
 	if (ruleset == Ruleset_None)
 	    for (j = Ruleset_None ; j < Ruleset_Count ; ++j)
-		strcpy(allresources[j][i], item);
+		allresources[j][i] = item;
     }
 
     fileclose(&file, NULL);
@@ -223,22 +234,22 @@ static int loadcolors(void)
     long	bkgnd, text, bold, dim;
     char       *end;
 
-    bkgnd = strtol(resources[RES_CLR_BKGND], &end, 16);
+    bkgnd = strtol(resources[RES_CLR_BKGND].str, &end, 16);
     if (*end || bkgnd < 0 || bkgnd > 0xFFFFFF) {
 	warn("rc: invalid color ID for background");
 	bkgnd = -1;
     }
-    text = strtol(resources[RES_CLR_TEXT], &end, 16);
+    text = strtol(resources[RES_CLR_TEXT].str, &end, 16);
     if (*end || text < 0 || text > 0xFFFFFF) {
 	warn("rc: invalid color ID for text");
 	text = -1;
     }
-    bold = strtol(resources[RES_CLR_BOLD], &end, 16);
+    bold = strtol(resources[RES_CLR_BOLD].str, &end, 16);
     if (*end || bold < 0 || bold > 0xFFFFFF) {
 	warn("rc: invalid color ID for bold text");
 	bold = -1;
     }
-    dim = strtol(resources[RES_CLR_DIM], &end, 16);
+    dim = strtol(resources[RES_CLR_DIM].str, &end, 16);
     if (*end || dim < 0 || dim > 0xFFFFFF) {
 	warn("rc: invalid color ID for dim text");
 	dim = -1;
@@ -257,13 +268,13 @@ static int loadimages(void)
 
     f = FALSE;
     path = getpathbuffer();
-    if (*resources[RES_IMG_TILES]) {
-	combinepath(path, resdir, resources[RES_IMG_TILES]);
+    if (*resources[RES_IMG_TILES].str) {
+	combinepath(path, resdir, resources[RES_IMG_TILES].str);
 	f = loadtileset(path, TRUE);
     }
     if (!f && resources != globalresources
-	   && *globalresources[RES_IMG_TILES]) {
-	combinepath(path, resdir, globalresources[RES_IMG_TILES]);
+	   && *globalresources[RES_IMG_TILES].str) {
+	combinepath(path, resdir, globalresources[RES_IMG_TILES].str);
 	f = loadtileset(path, TRUE);
     }
     free(path);
@@ -282,13 +293,13 @@ static int loadfont(void)
 
     f = FALSE;
     path = getpathbuffer();
-    if (*resources[RES_IMG_FONT]) {
-	combinepath(path, resdir, resources[RES_IMG_FONT]);
+    if (*resources[RES_IMG_FONT].str) {
+	combinepath(path, resdir, resources[RES_IMG_FONT].str);
 	f = loadfontfromfile(path, TRUE);
     }
     if (!f && resources != globalresources
-	   && *globalresources[RES_IMG_FONT]) {
-	combinepath(path, resdir, globalresources[RES_IMG_FONT]);
+	   && *globalresources[RES_IMG_FONT].str) {
+	combinepath(path, resdir, globalresources[RES_IMG_FONT].str);
 	f = loadfontfromfile(path, TRUE);
     }
     free(path);
@@ -300,19 +311,20 @@ static int loadfont(void)
 
 /* Load the list of unsolvable levels.
  */
-static int loadunslist(void)
+typedef int (*txtloader)(char const * fname);
+static int loadtxtresource(int resid, txtloader loadfunc)
 {
     char const *filename;
 
-    if (*resources[RES_TXT_UNSLIST])
-	filename = resources[RES_TXT_UNSLIST];
+    if (*resources[resid].str)
+	filename = resources[resid].str;
     else if (resources != globalresources
-			&& *globalresources[RES_TXT_UNSLIST])
-	filename = globalresources[RES_TXT_UNSLIST];
+			&& *globalresources[resid].str)
+	filename = globalresources[resid].str;
     else
 	return FALSE;
 
-    return loadunslistfromfile(filename);
+    return loadfunc(filename);
 }
 
 /* Load all of the sound resources.
@@ -327,13 +339,13 @@ static int loadsounds(void)
     count = 0;
     for (n = 0 ; n < SND_COUNT ; ++n) {
 	f = FALSE;
-	if (*resources[RES_SND_BASE + n]) {
-	    combinepath(path, resdir, resources[RES_SND_BASE + n]);
+	if (*resources[RES_SND_BASE + n].str) {
+	    combinepath(path, resdir, resources[RES_SND_BASE + n].str);
 	    f = loadsfxfromfile(n, path);
 	}
 	if (!f && resources != globalresources
-	       && *globalresources[RES_SND_BASE + n]) {
-	    combinepath(path, resdir, globalresources[RES_SND_BASE + n]);
+	       && *globalresources[RES_SND_BASE + n].str) {
+	    combinepath(path, resdir, globalresources[RES_SND_BASE + n].str);
 	    f = loadsfxfromfile(n, path);
 	}
 	if (f)
@@ -370,7 +382,8 @@ int initresources(void)
     resources = allresources[Ruleset_None];
     if (!readrcfile() || !loadcolors() || !loadfont())
 	return FALSE;
-    loadunslist();
+    loadtxtresource(RES_TXT_UNSLIST, loadunslistfromfile);
+    loadtxtresource(RES_TXT_MESSAGE, loadmessagesfromfile);
     return TRUE;
 }
 
@@ -379,7 +392,6 @@ int initresources(void)
 void freeallresources(void)
 {
     int	n;
-
     freefont();
     freetileset();
     clearunslist();
