@@ -4,13 +4,12 @@
  * License. No warranty. See COPYING for details.
  */
 
-#ifndef	_res_h_
-#define _res_h_
+#ifndef	HEADER_res_h_
+#define HEADER_res_h_
 
-/* Get and set the directory containing all the resource files.
+/* The directory containing all the resource files.
  */
-extern char const *getresdir(void);
-extern void setresdir(char const *dir);
+extern char	       *resdir;
 
 /* Parse the rc file and initialize the resources that are needed at
  * the start of the program (i.e., the font and color settings).
