@@ -4,20 +4,18 @@
  * License. No warranty. See COPYING for details.
  */
 
-#ifndef	_series_h_
-#define	_series_h_
+#ifndef	HEADER_series_h_
+#define	HEADER_series_h_
 
 #include	"defs.h"
 
-/* Get and set the series directory.
+/* The directory containing the series files.
  */
-extern char const *getseriesdir(void);
-extern void setseriesdir(char const *dir);
+extern char    *seriesdir;
 
-/* Get and set the series data directory.
+/* The directory containing the configured data files.
  */
-extern char const *getseriesdatdir(void);
-extern void setseriesdatdir(char const *dir);
+extern char    *seriesdatdir;
 
 /* Load all levels of the given series.
  */
@@ -39,9 +37,9 @@ extern void freeseriesdata(gameseries *series);
  * returned. FALSE is returned if no series files are found. An
  * unrecoverable error will cause the function to abort the program.
  */
-extern int createserieslist(char const *preferredfile,
-			    gameseries **pserieslist,
-			    int *pcount, tablespec *table);
+extern int createserieslist(char const *preferredfile, gameseries **pserieslist,
+		    	    int *pcount, mapfileinfo **pmflist, int *pmfcount,
+		            tablespec *table);
 
 /* Make an independent copy of a single gameseries structure from
  * a list obtained from createserieslist().
@@ -50,10 +48,11 @@ extern void getseriesfromlist(gameseries *dest,
 			      gameseries const *list, int index);
 
 /* Free the memory used by the table created in createserieslist().
- * Either serieslist or table can be NULL.
+ * The pointers can be NULL.
  */
-extern void freeserieslist(gameseries *serieslist, int count,
-			   tablespec *table);
+extern void freeserieslist(gameseries *list, int count,
+		    mapfileinfo *mflist, int mfcount,
+		    tablespec *table);
 
 /* A function for looking up a specific level in a series by number
  * and/or password. If number is -1, only the password will be
