@@ -25,7 +25,8 @@
  * HEADER
  *  0-3   signature bytes (35 33 9B 99)
  *   4    ruleset (1=Lynx, 2=MS)
- *  5-6   most recently visited level number
+ *   5    other options (currently always zero)
+ *   6    other options (currently always zero)
  *   7    count of bytes in remainder of header (currently always zero)
  *
  * After the header are level solutions, usually but not necessarily
@@ -118,10 +119,6 @@
  */
 #define	CSSIG		0x999B3335UL
 
-/* The three different modes that solutions files are opened with.
- */
-enum { F_READ, F_WRITE, F_MODIFY };
-
 /* Translate move directions between three-bit and four-bit
  * representations.
  *
@@ -149,25 +146,13 @@ static int const idxdir8[8] = {
 #define	dirtoindex(dir)		(diridx8[dir])
 #define	indextodir(dir)		(idxdir8[dir])
 
-/* The path of the directory containing the user's solution files.
- */
-static char const      *savedir = NULL;
-
 /* TRUE if file modification is prohibited.
  */
-static int		readonly = FALSE;
+int		readonly = FALSE;
 
-/* Getting and setting the save directory.
+/* The path of the directory containing the user's solution files.
  */
-char const *getsavedir(void)		{ return savedir; }
-void setsavedir(char const *dir)	{ savedir = dir; }
-
-/* Put the system in read-only mode.
- */
-void setreadonly(void)
-{
-    readonly = TRUE;
-}
+char	       *savedir = NULL;
 
 /*
  * Functions for manipulating move lists.
@@ -179,7 +164,7 @@ void initmovelist(actlist *list)
 {
     if (!list->allocated || !list->list) {
 	list->allocated = 16;
-	xalloc(list->list, list->allocated * sizeof *list->list);
+	x_alloc(list->list, list->allocated * sizeof *list->list);
     }
     list->count = 0;
 }
@@ -190,7 +175,7 @@ void addtomovelist(actlist *list, action move)
 {
     if (list->count >= list->allocated) {
 	list->allocated *= 2;
-	xalloc(list->list, list->allocated * sizeof *list->list);
+	x_alloc(list->list, list->allocated * sizeof *list->list);
     }
     list->list[list->count++] = move;
 }
@@ -203,7 +188,7 @@ void copymovelist(actlist *to, actlist const *from)
 	to->allocated = 16;
     while (to->allocated < from->count)
 	to->allocated *= 2;
-    xalloc(to->list, to->allocated * sizeof *to->list);
+    x_alloc(to->list, to->allocated * sizeof *to->list);
     to->count = from->count;
     if (from->count)
 	memcpy(to->list, from->list, from->count * sizeof *from->list);
@@ -558,14 +543,14 @@ static int writesolution(fileinfo *file, gamesetup const *game)
 
 /* Locate the solution file for the given data file and open it.
  */
-static int opensolutionfile(fileinfo *file, char const *datname, int mode)
+static int opensolutionfile(fileinfo *file, char const *datname, int writable)
 {
     static int	savedirchecked = FALSE;
     char       *buf = NULL;
     char const *filename;
     int		n;
 
-    if (mode != F_READ && readonly)
+    if (writable && readonly)
 	return FALSE;
 
     if (file->name) {
@@ -576,25 +561,25 @@ static int opensolutionfile(fileinfo *file, char const *datname, int mode)
 				  && tolower(datname[n - 2]) == 'a'
 				  && tolower(datname[n - 1]) == 't')
 	    n -= 4;
-	xalloc(buf, n + 5);
+	x_alloc(buf, n + 5);
 	memcpy(buf, datname, n);
 	memcpy(buf + n, ".tws", 5);
 	filename = buf;
     }
 
-    if (mode != F_WRITE) {
+    if (writable) {
 	if (!savedirchecked && savedir && *savedir && !haspathname(filename)) {
 	    savedirchecked = TRUE;
 	    if (!finddir(savedir)) {
-		setsavedir("");
+		*savedir = '\0';
 		fileerr(file, "can't access directory");
 	    }
 	}
     }
 
     n = openfileindir(file, savedir, filename,
-		      mode == F_WRITE ? "wb" : mode == F_MODIFY ? "r+b" : "rb",
-		      mode == F_WRITE ? "can't access file" : NULL);
+		      writable ? "wb" : "rb",
+		      writable ? "can't access file" : NULL);
     if (buf)
 	free(buf);
     return n;
@@ -604,38 +589,33 @@ static int opensolutionfile(fileinfo *file, char const *datname, int mode)
  */
 int readsolutions(gameseries *series)
 {
-    gamesetup	gametmp;
+    gamesetup	gametmp = {0};
     int		n;
 
     if (!series->savefile.name)
 	series->savefile.name = series->savefilename;
     if ((!series->savefile.name && (series->gsflags & GSF_NODEFAULTSAVE))
 		|| !opensolutionfile(&series->savefile,
-				     series->filebase, F_READ)) {
-	series->currentlevel = 0;
+				     series->filebase, FALSE)) {
+	series->solheaderflags = 0;
 	series->solheadersize = 0;
 	return TRUE;
     }
 
     if (!readsolutionheader(&series->savefile, series->ruleset,
-			    &series->currentlevel,
-			    &series->solheadersize, series->solheader)) {
-	series->currentlevel = 0;
-	series->solheadersize = 0;
+			    &series->solheaderflags,
+			    &series->solheadersize, series->solheader))
 	return FALSE;
-    }
 
-    memset(&gametmp, 0, sizeof gametmp);
     for (;;) {
 	if (!readsolution(&series->savefile, &gametmp))
 	    break;
 	if (gametmp.sgflags & SGF_SETNAME) {
 	    if (strcmp(gametmp.name, series->name)) {
-		errmsg(series->name, "ignoring solution file %s as it was"
-				     " recorded for a different level set: %s",
-		       series->savefile.name, gametmp.name);
+		warn("%s: solution file %s was recorded for a different level"
+		    " set (%s); continuing anyway, but disabling saving",
+		    series->name, series->savefile.name, gametmp.name);
 		series->gsflags |= GSF_NOSAVING;
-		return FALSE;
 	    }
 	    continue;
 	}
@@ -676,11 +656,11 @@ int savesolutions(gameseries *series)
 	series->savefile.name = series->savefilename;
     if (!series->savefile.name && (series->gsflags & GSF_NODEFAULTSAVE))
 	return TRUE;
-    if (!opensolutionfile(&series->savefile, series->filebase, F_WRITE))
+    if (!opensolutionfile(&series->savefile, series->filebase, TRUE))
 	return FALSE;
 
     if (!writesolutionheader(&series->savefile, series->ruleset,
-			     series->currentlevel,
+			     series->solheaderflags,
 			     series->solheadersize, series->solheader))
 	return fileerr(&series->savefile,
 		       "saved-game file has become corrupted!");
@@ -697,27 +677,6 @@ int savesolutions(gameseries *series)
     return TRUE;
 }
 
-/* Write out just the current level number to the existing solution
- * file.
- */
-int savesolutionlevel(gameseries *series)
-{
-    int f;
-
-    if (readonly)
-	return FALSE;
-    if (!series->savefile.name)
-	series->savefile.name = series->savefilename;
-    if (!series->savefile.name && (series->gsflags & GSF_NODEFAULTSAVE))
-	return FALSE;
-    if (!opensolutionfile(&series->savefile, series->filebase, F_MODIFY))
-	return FALSE;
-    f = fileskip(&series->savefile, 5, NULL)
-	    && filewriteint16(&series->savefile, series->currentlevel, NULL);
-    fileclose(&series->savefile, NULL);
-    return f;
-}
-
 /* Free all memory allocated for storing the game's solutions, and mark
  * the levels as being unsolved.
  */
@@ -734,7 +693,7 @@ void clearsolutions(gameseries *series)
 	game->solutiondata = NULL;
     }
     series->solheadersize = 0;
-    series->currentlevel = 0;
+    series->solheaderflags = 0;
     fileclose(&series->savefile, NULL);
     clearfileinfo(&series->savefile);
 }
@@ -800,14 +759,14 @@ typedef	struct solutiondata {
  * structure, then add it to the pool of filenames, prefixed with
  * "1-". This function is a callback for findfiles().
  */
-static int getsolutionfile(char *filename, void *data)
+static int getsolutionfile(char const *filename, void *data)
 {
     solutiondata       *sdata = data;
     int			n;
 
     if (!memcmp(filename, sdata->prefix, sdata->prefixlen)) {
 	n = strlen(filename) + 1;
-	xalloc(sdata->pool, sdata->allocated + n + 2);
+	x_alloc(sdata->pool, sdata->allocated + n + 2);
 	sdata->pool[sdata->allocated++] = '1';
 	sdata->pool[sdata->allocated++] = '-';
 	memcpy(sdata->pool + sdata->allocated, filename, n);
@@ -857,7 +816,7 @@ int createsolutionfilelist(gameseries const *series, int morethanone,
 	memerrexit();
     table->rows = s.count + 1;
     table->cols = 2;
-    table->sep = 1;
+    table->sep = 4;
     table->collapse = 1;
     table->items[0] = "2-Select a solution file";
     offset = 0;
@@ -881,7 +840,7 @@ void freesolutionfilelist(char const **filelist, tablespec *table)
 {
     free(filelist);
     if (table) {
-	free(table->items[2]);
+	free((void*)table->items[2]);
 	free(table->items);
     }
 }
