@@ -1,23 +1,32 @@
 /* solution.c: Functions for reading and writing the solution files.
  *
- * Copyright (C) 2001-2006 by Brian Raiter, under the GNU General Public
- * License. No warranty. See COPYING for details.
+ * Copyright (C) 2001-2010 by Brian Raiter and Madhav Shanbhag,
+ * under the GNU General Public License. No warranty. See COPYING for details.
  */
 
-#ifndef	_solution_h_
-#define	_solution_h_
+#ifndef	HEADER_solution_h_
+#define	HEADER_solution_h_
 
 #include	"defs.h"
 #include	"fileio.h"
 
-/* Get and set the directory containing the user's solution files.
+/* A structure holding all the data needed to reconstruct a solution.
  */
-extern char const *getsavedir(void);
-extern void setsavedir(char const *dir);
+typedef	struct solutioninfo {
+    actlist		moves;		/* the actual moves of the solution */
+    unsigned long	rndseed;	/* the PRNG's initial seed */
+    unsigned long	flags;		/* other flags (currently unused) */
+    unsigned char	rndslidedir;	/* random slide's initial direction */
+    signed char		stepping;	/* the timer offset */
+} solutioninfo;
 
-/* No file modification will be done after this function is called.
+/* The path of the directory containing the user's solution files.
  */
-extern void setreadonly(void);
+extern char    *savedir;
+
+/* No file modification will be done unless this variable is FALSE.
+ */
+extern int	readonly;
 
 /* Initialize or reinitialize list as empty.
  */
@@ -74,12 +83,6 @@ extern void clearsolutions(gameseries *series);
  */
 extern int loadsolutionsetname(char const *filename, char *buffer);
 
-/* Write out just the current level for the given series. TRUE is
- * returned if the level number was successfully saved. No error is
- * displayed if the value cannot be written.
- */
-extern int savesolutionlevel(gameseries *series);
-
 /* Produce a list of available solution files associated with the
  * given series (i.e. that have the name of the series as their
  * prefix). An array of filenames is returned through pfilelist, the
